@@ -7,7 +7,7 @@
  *  after which e should be just a1^2. */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "times.h"
@@ -64,10 +64,10 @@ unsigned time_dennyfliegner()
 	vector<double> times;
 	timer breitling;
 	
-	sizes.push_back(50);
 	sizes.push_back(100);
 	sizes.push_back(200);
 	sizes.push_back(400);
+	sizes.push_back(800);
 	
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i) {
 		breitling.start();
@@ -85,10 +85,10 @@ unsigned time_dennyfliegner()
 	// print the report:
 	cout << endl << "	size:  ";
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i)
-		cout << '\t' << (*i);
+		cout << '\t' << *i;
 	cout << endl << "	time/s:";
 	for (vector<double>::iterator i=times.begin(); i!=times.end(); ++i)
-		cout << '\t' << int(1000*(*i))*0.001;
+		cout << '\t' << *i;
 	cout << endl;
 	
 	return result;
