@@ -4,7 +4,7 @@
  *  by Robert H. Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "times.h"
@@ -45,7 +45,7 @@ static unsigned test(unsigned n)
 	Hinv = H.inverse();
 	cout << ". passed ";
 	clog << "(no output)" << endl;
-	cout << int(1000*cartier.read())*0.001 << 's' << endl;
+	cout << cartier.read() << 's' << endl;
 	
 	// check result:
 	name = (n==40?'J':(n==70?'L':'?'));
@@ -75,7 +75,7 @@ static unsigned test(unsigned n)
 		cout << ". failed ";
 		++result;
 	}
-	cout << int(1000*cartier.read())*0.001 << 's' << endl;
+	cout << cartier.read() << 's' << endl;
 	return result;
 }
 
