@@ -4,7 +4,7 @@
  *  Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -174,7 +174,7 @@ unsigned time_lw_M2()
 		} else {
 			cout << " failed ";
 		}
-		cout << int(1000*(time/count))*0.001 << 's' << endl;
+		cout << time/count << 's' << endl;
 	} else {
 		cout << " disabled" << endl;
 		clog << "(no output)" << endl;
