@@ -66,12 +66,23 @@ public:
 		property_info() {}
 		property_info(property_type t, const std::string &n, unsigned c = 1) : type(t), name(n), count(c) {}
 
-		property_type type;	/**< Data type of property. */
+		property_type type; /**< Data type of property. */
 		std::string name;   /**< Name of property. */
 		unsigned count;     /**< Number of occurrences. */
 	};
 	typedef std::vector<property_info> propinfovector;
 
+	/** Archived property (data type, name and associated data) */
+	struct property {
+		property() {}
+		property(archive_atom n, property_type t, unsigned v) : type(t), name(n), value(v) {}
+
+		property_type type; /**< Data type of property. */
+		archive_atom name;  /**< Name of property. */
+		unsigned value;     /**< Stored value. */
+	};
+	typedef std::vector<property>::const_iterator archive_node_cit;
+
 	archive_node() : a(*dummy_ar_creator()), has_expression(false) {} // hack for cint which always requires a default constructor
 	archive_node(archive &ar) : a(ar), has_expression(false) {}
 	archive_node(archive &ar, const ex &expr);
@@ -102,10 +113,20 @@ public:
 	 *  @return "true" if property was found, "false" otherwise */
 	bool find_string(const std::string &name, std::string &ret, unsigned index = 0) const;
 
+	/** Find the location in the vector of properties of the first/last
+    *  property with a given name. */
+	archive_node_cit find_first(const std::string &name) const;
+	archive_node_cit find_last(const std::string &name) const;
+
 	/** Retrieve property of type "ex" from node.
 	 *  @return "true" if property was found, "false" otherwise */
 	bool find_ex(const std::string &name, ex &ret, lst &sym_lst, unsigned index = 0) const;
 
+	/** Retrieve property of type "ex" from the node if it is known
+    *  that this node in fact contains such a property at the given
+    *  location. This is much more efficient than the preceding function. */
+	void find_ex_by_loc(archive_node_cit loc, ex &ret, lst &sym_lst) const;
+
 	/** Retrieve property of type "ex" from node, returning the node of
 	 *  the sub-expression. */
 	const archive_node &find_ex_node(const std::string &name, unsigned index = 0) const;
@@ -124,16 +145,6 @@ public:
 private:
 	static archive* dummy_ar_creator();
 
-	/** Archived property (data type, name and associated data) */
-	struct property {
-		property() {}
-		property(archive_atom n, property_type t, unsigned v) : type(t), name(n), value(v) {}
-
-		property_type type; /**< Data type of property. */
-		archive_atom name;  /**< Name of property. */
-		unsigned value; /**< Stored value. */
-	};
-
 	/** Reference to the archive to which this node belongs. */
 	archive &a;
 
@@ -217,8 +228,8 @@ private:
 		archived_ex() {}
 		archived_ex(archive_atom n, archive_node_id node) : name(n), root(node) {}
 
-		archive_atom name;		/**< Name of expression. */
-		archive_node_id root;	/**< ID of root node. */
+		archive_atom name;     /**< Name of expression. */
+		archive_node_id root;  /**< ID of root node. */
 	};
 
 	/** Vector of archived expression descriptors. */
@@ -231,6 +242,11 @@ public:
 private:
 	/** Vector of atomized strings (using a vector allows faster unarchiving). */
 	mutable std::vector<std::string> atoms;
+	/** The map of from strings to indices of the atoms vectors allows for
+	 *  faster archiving.
+	 */
+	typedef std::map<std::string, archive_atom>::const_iterator inv_at_cit;
+	mutable std::map<std::string, archive_atom> inverse_atoms;
 
 	/** Map of stored expressions to nodes for faster archiving */
 	typedef std::map<ex, archive_node_id, ex_is_less>::iterator mapit;
