@@ -20,6 +20,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <stdexcept>
+
 #include "clifford.h"
 
 #include "ex.h"
@@ -44,6 +46,8 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(clifford, indexed,
   print_func<print_dflt>(&clifford::do_print_dflt).
   print_func<print_latex>(&clifford::do_print_latex))
 
+const tinfo_static_t clifford::return_type_tinfo_static[256] = {{}};
+
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(diracone, tensor,
   print_func<print_dflt>(&diracone::do_print).
   print_func<print_latex>(&diracone::do_print_latex))
@@ -72,15 +76,9 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(diracgammaR, tensor,
 // default constructors
 //////////
 
-static ex default_metric()
+clifford::clifford() : representation_label(0), metric(0), commutator_sign(-1)
 {
-	static ex m = (new minkmetric)->setflag(status_flags::dynallocated);
-	return m;
-}
-
-clifford::clifford() : representation_label(0), metric(default_metric())
-{
-	tinfo_key = TINFO_clifford;
+	tinfo_key = &clifford::tinfo_static;
 }
 
 DEFAULT_CTOR(diracone)
@@ -97,29 +95,29 @@ DEFAULT_CTOR(diracgammaR)
 /** Construct object without any indices. This constructor is for internal
  *  use only. Use the dirac_ONE() function instead.
  *  @see dirac_ONE */
-clifford::clifford(const ex & b, unsigned char rl) : inherited(b), representation_label(rl), metric(0)
+clifford::clifford(const ex & b, unsigned char rl) : inherited(b), representation_label(rl), metric(0), commutator_sign(-1)
 {
-	tinfo_key = TINFO_clifford;
+	tinfo_key = &clifford::tinfo_static;
 }
 
 /** Construct object with one Lorentz index. This constructor is for internal
  *  use only. Use the clifford_unit() or dirac_gamma() functions instead.
  *  @see clifford_unit
  *  @see dirac_gamma */
-clifford::clifford(const ex & b, const ex & mu, const ex & metr, unsigned char rl) : inherited(b, mu), representation_label(rl), metric(metr)
+clifford::clifford(const ex & b, const ex & mu, const ex & metr, unsigned char rl, int comm_sign) : inherited(b, mu), representation_label(rl), metric(metr), commutator_sign(comm_sign)
 {
 	GINAC_ASSERT(is_a<varidx>(mu));
-	tinfo_key = TINFO_clifford;
+	tinfo_key = &clifford::tinfo_static;
 }
 
-clifford::clifford(unsigned char rl, const ex & metr, const exvector & v, bool discardable) : inherited(not_symmetric(), v, discardable), representation_label(rl), metric(metr)
+clifford::clifford(unsigned char rl, const ex & metr, int comm_sign, const exvector & v, bool discardable) : inherited(not_symmetric(), v, discardable), representation_label(rl), metric(metr), commutator_sign(comm_sign)
 {
-	tinfo_key = TINFO_clifford;
+	tinfo_key = &clifford::tinfo_static;
 }
 
-clifford::clifford(unsigned char rl, const ex & metr, std::auto_ptr<exvector> vp) : inherited(not_symmetric(), vp), representation_label(rl), metric(metr)
+clifford::clifford(unsigned char rl, const ex & metr, int comm_sign, std::auto_ptr<exvector> vp) : inherited(not_symmetric(), vp), representation_label(rl), metric(metr), commutator_sign(comm_sign)
 {
-	tinfo_key = TINFO_clifford;
+	tinfo_key = &clifford::tinfo_static;
 }
 
 //////////
@@ -132,6 +130,8 @@ clifford::clifford(const archive_node & n, lst & sym_lst) : inherited(n, sym_lst
 	n.find_unsigned("label", rl);
 	representation_label = rl;
 	n.find_ex("metric", metric, sym_lst);
+	n.find_unsigned("commutator_sign+1", rl);
+	commutator_sign = rl - 1;
 }
 
 void clifford::archive(archive_node & n) const
@@ -139,6 +139,7 @@ void clifford::archive(archive_node & n) const
 	inherited::archive(n);
 	n.add_unsigned("label", representation_label);
 	n.add_ex("metric", metric);
+	n.add_unsigned("commutator_sign+1", commutator_sign+1);
 }
 
 DEFAULT_UNARCHIVE(clifford)
@@ -149,23 +150,85 @@ DEFAULT_ARCHIVING(diracgamma5)
 DEFAULT_ARCHIVING(diracgammaL)
 DEFAULT_ARCHIVING(diracgammaR)
 
-//////////
-// functions overriding virtual functions from base classes
-//////////
 
-ex clifford::get_metric(const ex & i, const ex & j) const
+ex clifford::get_metric(const ex & i, const ex & j, bool symmetrised) const
 {
-	return indexed(metric, symmetric2(), i, j);
+	if (is_a<indexed>(metric)) {
+		if (symmetrised && !(ex_to<symmetry>(ex_to<indexed>(metric).get_symmetry()).has_symmetry())) {
+			if (is_a<matrix>(metric.op(0))) {
+				return indexed((ex_to<matrix>(metric.op(0)).add(ex_to<matrix>(metric.op(0)).transpose())).mul(numeric(1, 2)),
+				               symmetric2(), i, j);
+			} else {
+				return simplify_indexed(indexed(metric.op(0)*_ex1_2, i, j) + indexed(metric.op(0)*_ex1_2, j, i));
+			}
+		} else {
+			return metric.subs(lst(metric.op(1) == i, metric.op(2) == j), subs_options::no_pattern);
+		}
+	} else {
+		exvector indices = metric.get_free_indices();
+		if (symmetrised)
+			return _ex1_2*simplify_indexed(metric.subs(lst(indices[0] == i, indices[1] == j), subs_options::no_pattern)
+									+ metric.subs(lst(indices[0] == j, indices[1] == i), subs_options::no_pattern));
+		else
+			return metric.subs(lst(indices[0] == i, indices[1] == j), subs_options::no_pattern);
+	}
 }
 
 bool clifford::same_metric(const ex & other) const
 {
-	if (is_a<clifford>(other)) {
-		return get_metric().is_equal(ex_to<clifford>(other).get_metric());
-	} else if (is_a<indexed>(other)) {
-		return get_metric(other.op(1), other.op(2)).is_equal(other);
-	} else
-		return false;
+	ex metr;
+	if (is_a<clifford>(other)) 
+		metr = ex_to<clifford>(other).get_metric();
+	else 
+		metr = other;
+
+	if (is_a<indexed>(metr))
+		return metr.op(0).is_equal(get_metric().op(0));
+	else {
+		exvector indices = metr.get_free_indices();
+		return  (indices.size() == 2) 
+			&& simplify_indexed(get_metric(indices[0], indices[1])-metr).is_zero();
+	}
+}
+
+//////////
+// functions overriding virtual functions from base classes
+//////////
+
+ex clifford::op(size_t i) const
+{
+	GINAC_ASSERT(i<nops());
+	if (nops()-i == 1)
+		return representation_label;
+	else 
+		return inherited::op(i);
+}
+
+ex & clifford::let_op(size_t i)
+{
+        GINAC_ASSERT(i<nops());
+
+	static ex rl = numeric(representation_label);
+        ensure_if_modifiable();
+	if (nops()-i == 1)
+		return rl;
+	else 
+		return inherited::let_op(i);
+}
+
+ex clifford::subs(const exmap & m, unsigned options) const
+{
+	ex subsed = inherited::subs(m, options);
+	if(is_a<clifford>(subsed)) {
+		ex prevmetric = ex_to<clifford>(subsed).metric;
+		ex newmetric = prevmetric.subs(m, options);
+		if(!are_ex_trivially_equal(prevmetric, newmetric)) {
+			clifford c = ex_to<clifford>(subsed);
+			c.metric = newmetric;
+			subsed = c;
+		}
+	}
+	return subsed;
 }
 
 int clifford::compare_same_type(const basic & other) const
@@ -186,7 +249,7 @@ bool clifford::match_same_type(const basic & other) const
 	GINAC_ASSERT(is_a<clifford>(other));
 	const clifford &o = static_cast<const clifford &>(other);
 
-	return (representation_label == o.representation_label) && same_metric(o);
+	return ((representation_label == o.representation_label) && (commutator_sign == o.get_commutator_sign()) && same_metric(o));
 }
 
 static bool is_dirac_slash(const ex & seq0)
@@ -202,8 +265,21 @@ void clifford::do_print_dflt(const print_dflt & c, unsigned level) const
 	if (is_dirac_slash(seq[0])) {
 		seq[0].print(c, precedence());
 		c.s << "\\";
-	} else
-		this->print_dispatch<inherited>(c, level);
+	} else { // We do not print representation label if it is 0
+		if (representation_label == 0) {
+			this->print_dispatch<inherited>(c, level);
+		} else { // otherwise we put it before indices in square brackets; the code is borrowed from indexed.cpp 
+			if (precedence() <= level) {
+				c.s << '(';
+			}
+			seq[0].print(c, precedence());
+			c.s << '[' << int(representation_label) << ']';
+			printindices(c, level);
+			if (precedence() <= level) {
+				c.s << ')';
+			}
+		}
+	}
 }
 
 void clifford::do_print_latex(const print_latex & c, unsigned level) const
@@ -237,7 +313,7 @@ DEFAULT_PRINT_LATEX(diracgammaR, "gammaR", "{\\gamma_R}")
 static void base_and_index(const ex & c, ex & b, ex & i)
 {
 	GINAC_ASSERT(is_a<clifford>(c));
-	GINAC_ASSERT(c.nops() == 2);
+	GINAC_ASSERT(c.nops() == 2+1);
 
 	if (is_a<cliffordunit>(c.op(0))) { // proper dirac gamma object or clifford unit
 		i = c.op(1);
@@ -371,21 +447,6 @@ bool diracgamma::contract_with(exvector::iterator self, exvector::iterator other
 	return false;
 }
 
-/** An utility function looking for a given metric within an exvector,
- *  used in cliffordunit::contract_with(). */
-static int find_same_metric(exvector & v, ex & c)
-{
-	for (int i=0; i<v.size();i++) {
-		if (!is_a<clifford>(v[i]) && is_a<indexed>(v[i])
-		    && ex_to<clifford>(c).same_metric(v[i]) 
-		    && (ex_to<varidx>(c.op(1)) == ex_to<indexed>(v[i]).get_indices()[0]
-			|| ex_to<varidx>(c.op(1)).toggle_variance() == ex_to<indexed>(v[i]).get_indices()[0])) {
-			return ++i; // next to found
-		}
-	}
-	return 0; //nothing found
-}
-
 /** Contraction of a Clifford unit with something else. */
 bool cliffordunit::contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const
 {
@@ -402,71 +463,52 @@ bool cliffordunit::contract_with(exvector::iterator self, exvector::iterator oth
 		    && unit.same_metric(*other))
 			return false;
 
-		// Find if a previous contraction produces the square of self
-		int prev_square = find_same_metric(v, self[0]);
-		varidx d((new symbol)->setflag(status_flags::dynallocated), ex_to<idx>(self->op(1)).get_dim());
-		ex squared_metric = unit.get_metric(self->op(1), d) * unit.get_metric(d.toggle_variance(), other->op(1));
+		exvector::iterator before_other = other - 1;
+		ex mu = self->op(1);
+		ex mu_toggle = other->op(1);
+		ex alpha = before_other->op(1);
 
 		// e~mu e.mu = Tr ONE
 		if (other - self == 1) {
-			if (prev_square != 0) {
-				*self = squared_metric;
-				v[prev_square-1] = _ex1;
-			} else
-				*self = unit.get_metric(self->op(1), other->op(1));
+			*self = unit.get_metric(mu, mu_toggle, true);
 			*other = dirac_ONE(rl);
 			return true;
 
-		// e~mu e~alpha e.mu = (2e~alpha^2-Tr) e~alpha
-		} else if (other - self == 2
-		        && is_a<clifford>(self[1])) {
+		} else if (other - self == 2) {
+			if (is_a<clifford>(*before_other) && ex_to<clifford>(*before_other).get_representation_label() == rl) {
+				// e~mu e~alpha e.mu = 2*e~mu B(alpha, mu.toggle_variance())-Tr(B) e~alpha
+				*self = 2 * (*self) * unit.get_metric(alpha, mu_toggle, true) - unit.get_metric(mu, mu_toggle, true) * (*before_other);
+				*before_other = _ex1;
+				*other = _ex1;
+				return true;
 
-			const ex & ia = self[1].op(1);
-			const ex & ib = self[1].op(1);
-			if (is_a<tensmetric>(unit.get_metric()))
-				*self = 2 - unit.get_metric(self->op(1), other->op(1));
-			else if (prev_square != 0) {
-				*self = 2-squared_metric;
-				v[prev_square-1] = _ex1;
-			} else 
-				*self = 2*unit.get_metric(ia, ib) - unit.get_metric(self->op(1), other->op(1));
-			*other = _ex1;
-			return true;
-
-		// e~mu S e~alpha e.mu = 2 e~alpha^3 S - e~mu S e.mu e~alpha
+			} else {
+				// e~mu S e.mu = Tr S ONE
+				*self = unit.get_metric(mu, mu_toggle, true);
+				*other = dirac_ONE(rl);
+				return true;
+			}
+		} else {
+		// e~mu S e~alpha e.mu = 2 e~mu S B(alpha, mu.toggle_variance()) - e~mu S e.mu e~alpha
 		// (commutate contracted indices towards each other, simplify_indexed()
 		// will re-expand and re-run the simplification)
-		} else {
-			exvector::iterator it = self + 1, next_to_last = other - 1;
-			while (it != other) {
-				if (!is_a<clifford>(*it))
-					return false;
-				++it;
+			if (std::find_if(self + 1, other, is_not_a_clifford()) != other) {
+				return false;
 			}
+			
+			ex S = ncmul(exvector(self + 1, before_other), true);
 
-			it = self + 1;
-			ex S = _ex1;
-			while (it != next_to_last) {
-				S *= *it;
-				*it++ = _ex1;
+			if (is_a<clifford>(*before_other) && ex_to<clifford>(*before_other).get_representation_label() == rl) {
+				*self = 2 * (*self) * S * unit.get_metric(alpha, mu_toggle, true) - (*self) * S * (*other) * (*before_other);
+			} else {
+				// simply commutes
+				*self = (*self) * S * (*other) * (*before_other);
 			}
-
-			const ex & ia = next_to_last->op(1);
-			const ex & ib = next_to_last->op(1);
-			if (is_a<tensmetric>(unit.get_metric()))
-				*self = 2 * (*next_to_last) * S - (*self) * S * (*other) * (*next_to_last);
-			else if (prev_square != 0) {
-				*self = 2 * (*next_to_last) * S  - (*self) * S * (*other) * (*next_to_last)*unit.get_metric(self->op(1),self->op(1));
-				v[prev_square-1] = _ex1;
-			} else 
-				*self = 2 * (*next_to_last) * S* unit.get_metric(ia,ib) - (*self) * S * (*other) * (*next_to_last);
-			*next_to_last = _ex1;
-			*other = _ex1;
+				
+			std::fill(self + 1, other + 1, _ex1);
 			return true;
 		}
-
-	} 
-
+	}
 	return false;
 }
 
@@ -575,12 +617,14 @@ ex clifford::eval_ncmul(const exvector & v) const
 			bool a_is_cliffordunit = is_a<cliffordunit>(ag);
 			bool b_is_cliffordunit =  is_a<cliffordunit>(bg);
 
-			if (a_is_cliffordunit && b_is_cliffordunit && ex_to<clifford>(a).same_metric(b)) {
-
+			if (a_is_cliffordunit && b_is_cliffordunit && ex_to<clifford>(a).same_metric(b)
+				&& (ex_to<clifford>(a).get_commutator_sign() == -1)) {
+				// This is done only for Clifford algebras 
+				
 				const ex & ia = a.op(1);
 				const ex & ib = b.op(1);
 				if (ia.is_equal(ib)) { // gamma~alpha gamma~alpha -> g~alpha~alpha
-					a = ex_to<clifford>(a).get_metric(ia, ib);
+					a = ex_to<clifford>(a).get_metric(ia, ib, true);
 					b = dirac_ONE(representation_label);
 					something_changed = true;
 				}
@@ -630,7 +674,7 @@ ex clifford::eval_ncmul(const exvector & v) const
 	}
 
 	if (s.empty())
-		return clifford(diracone(), representation_label) * sign;
+		return dirac_ONE(representation_label) * sign;
 	if (something_changed)
 		return reeval_ncmul(s) * sign;
 	else
@@ -639,12 +683,12 @@ ex clifford::eval_ncmul(const exvector & v) const
 
 ex clifford::thiscontainer(const exvector & v) const
 {
-	return clifford(representation_label, get_metric(), v);
+	return clifford(representation_label, metric, commutator_sign, v);
 }
 
 ex clifford::thiscontainer(std::auto_ptr<exvector> vp) const
 {
-	return clifford(representation_label, get_metric(), vp);
+	return clifford(representation_label, metric, commutator_sign, vp);
 }
 
 ex diracgamma5::conjugate() const
@@ -676,15 +720,38 @@ ex clifford_unit(const ex & mu, const ex & metr, unsigned char rl)
 {
 	static ex unit = (new cliffordunit)->setflag(status_flags::dynallocated);
 
-	if (!is_a<varidx>(mu))
-		throw(std::invalid_argument("index of Clifford unit must be of type varidx"));
+	if (!is_a<idx>(mu))
+		throw(std::invalid_argument("clifford_unit(): index of Clifford unit must be of type idx or varidx"));
 
-	if (is_a<indexed>(metr))
-		return clifford(unit, mu, metr.op(0), rl);
-	else if(is_a<tensmetric>(metr) || is_a<matrix>(metr)) 
+	exvector indices = metr.get_free_indices();
+
+	if (indices.size() == 2) {
 		return clifford(unit, mu, metr, rl);
-	else
-		throw(std::invalid_argument("metric for Clifford unit must be of type indexed, tensormetric or matrix"));
+	} else if (is_a<matrix>(metr)) {
+		matrix M = ex_to<matrix>(metr);
+		unsigned n = M.rows();
+		bool symmetric = true;
+
+		static idx xi((new symbol)->setflag(status_flags::dynallocated), n),
+			chi((new symbol)->setflag(status_flags::dynallocated), n);
+		if ((n ==  M.cols()) && (n == ex_to<idx>(mu).get_dim())) {
+			for (unsigned i = 0; i < n; i++) {
+				for (unsigned j = i+1; j < n; j++) {
+					if (M(i, j) != M(j, i)) {
+						symmetric = false;
+					}
+				}
+			}
+			return clifford(unit, mu, indexed(metr, symmetric?symmetric2():not_symmetric(), xi, chi), rl);
+		} else {
+			throw(std::invalid_argument("clifford_unit(): metric for Clifford unit must be a square matrix with the same dimensions as index"));
+		}
+	} else if (indices.size() == 0) { // a tensor or other expression without indices
+		static varidx xi((new symbol)->setflag(status_flags::dynallocated), ex_to<idx>(mu).get_dim()),
+			chi((new symbol)->setflag(status_flags::dynallocated), ex_to<idx>(mu).get_dim());
+		return clifford(unit, mu, indexed(metr, xi, chi), rl);
+	}  else 
+		throw(std::invalid_argument("clifford_unit(): metric for Clifford unit must be of type tensor, matrix or an expression with two free indices"));
 }
 
 ex dirac_gamma(const ex & mu, unsigned char rl)
@@ -692,9 +759,11 @@ ex dirac_gamma(const ex & mu, unsigned char rl)
 	static ex gamma = (new diracgamma)->setflag(status_flags::dynallocated);
 
 	if (!is_a<varidx>(mu))
-		throw(std::invalid_argument("index of Dirac gamma must be of type varidx"));
+		throw(std::invalid_argument("dirac_gamma(): index of Dirac gamma must be of type varidx"));
 
-	return clifford(gamma, mu, default_metric(), rl);
+	static varidx xi((new symbol)->setflag(status_flags::dynallocated), ex_to<varidx>(mu).get_dim()),
+		chi((new symbol)->setflag(status_flags::dynallocated), ex_to<varidx>(mu).get_dim());
+	return clifford(gamma, mu, indexed((new minkmetric)->setflag(status_flags::dynallocated), symmetric2(), xi, chi), rl);
 }
 
 ex dirac_gamma5(unsigned char rl)
@@ -720,28 +789,25 @@ ex dirac_slash(const ex & e, const ex & dim, unsigned char rl)
 	// Slashed vectors are actually stored as a clifford object with the
 	// vector as its base expression and a (dummy) index that just serves
 	// for storing the space dimensionality
-	return clifford(e, varidx(0, dim), default_metric(), rl);
-}
 
-/** Check whether a given tinfo key (as returned by return_type_tinfo()
- *  is that of a clifford object with the specified representation label. */
-static bool is_clifford_tinfo(unsigned ti, unsigned char rl)
-{
-	return ti == (TINFO_clifford + rl);
+	static varidx xi((new symbol)->setflag(status_flags::dynallocated), dim),
+		chi((new symbol)->setflag(status_flags::dynallocated), dim);
+   return clifford(e, varidx(0, dim), indexed((new minkmetric)->setflag(status_flags::dynallocated), symmetric2(), xi, chi), rl);
 }
 
 /** Check whether a given tinfo key (as returned by return_type_tinfo()
  *  is that of a clifford object (with an arbitrary representation label). */
-static bool is_clifford_tinfo(unsigned ti)
+bool is_clifford_tinfo(tinfo_t ti)
 {
-	return (ti & ~0xff) == TINFO_clifford;
+	p_int start_loc=(p_int)&clifford::return_type_tinfo_static;
+	return (p_int)ti>=start_loc && (p_int)ti<start_loc+256;
 }
 
 /** Extract representation label from tinfo key (as returned by
  *  return_type_tinfo()). */
-static unsigned char get_representation_label(unsigned ti)
+static unsigned char get_representation_label(tinfo_t ti)
 {
-	return ti & 0xff;
+	return (unsigned char)((p_int)ti-(p_int)&clifford::return_type_tinfo_static);
 }
 
 /** Take trace of a string of an even number of Dirac gammas given a vector
@@ -944,7 +1010,7 @@ ex canonicalize_clifford(const ex & e_)
 	pointer_to_map_function fcn(canonicalize_clifford);
 
 	if (is_a<matrix>(e_)    // || is_a<pseries>(e) || is_a<integral>(e)
-		|| is_a<lst>(e_)) {
+		|| e_.info(info_flags::list)) {
 		return e_.map(fcn);
 	} else {
 		ex e=simplify_indexed(e_);
@@ -978,18 +1044,21 @@ ex canonicalize_clifford(const ex & e_)
 				exvector::iterator it = v.begin(), next_to_last = v.end() - 1;
 				if (is_a<diracgamma5>(it->op(0)) || is_a<diracgammaL>(it->op(0)) || is_a<diracgammaR>(it->op(0)))
 					++it;
+
 				while (it != next_to_last) {
 					if (it[0].compare(it[1]) > 0) {
+
 						ex save0 = it[0], save1 = it[1];
 						ex b1, i1, b2, i2;
 						base_and_index(it[0], b1, i1);
 						base_and_index(it[1], b2, i2);
-						it[0] = (ex_to<clifford>(save0).get_metric(i1, i2) * b1 * b2).simplify_indexed();
-						it[1] = v.size() == 2 ? _ex2 * dirac_ONE(ex_to<clifford>(it[1]).get_representation_label()) : _ex2;
+						// for Clifford algebras (commutator_sign == -1) metric should be symmetrised
+						it[0] = (ex_to<clifford>(save0).get_metric(i1, i2, ex_to<clifford>(save0).get_commutator_sign() == -1) * b1 * b2).simplify_indexed();
+						it[1] = v.size() ? _ex2 * dirac_ONE(ex_to<clifford>(save0).get_representation_label()) : _ex2;
 						ex sum = ncmul(v);
 						it[0] = save1;
 						it[1] = save0;
-						sum -= ncmul(v, true);
+						sum += ex_to<clifford>(save0).get_commutator_sign() * ncmul(v, true);
 						i->second = canonicalize_clifford(sum);
 						goto next_sym;
 					}
@@ -1008,7 +1077,7 @@ ex clifford_prime(const ex & e)
 	if (is_a<clifford>(e) && is_a<cliffordunit>(e.op(0))) {
 		return -e;
 	} else if (is_a<add>(e) || is_a<ncmul>(e) || is_a<mul>(e) //|| is_a<pseries>(e) || is_a<integral>(e)
-			   || is_a<matrix>(e) || is_a<lst>(e)) {
+			   || is_a<matrix>(e) || e.info(info_flags::list)) {
 		return e.map(fcn);
 	} else if (is_a<power>(e)) {
 		return pow(clifford_prime(e.op(0)), e.op(1));
@@ -1016,46 +1085,80 @@ ex clifford_prime(const ex & e)
 		return e;
 }
 
-ex remove_dirac_ONE(const ex & e)
+ex remove_dirac_ONE(const ex & e, unsigned char rl, unsigned options)
 {
-	return remove_dirac_ONE(e, 0);
+	pointer_to_map_function_2args<unsigned char, unsigned> fcn(remove_dirac_ONE, rl, options | 1);
+	bool need_reevaluation = false;
+	ex e1 = e;
+	if (! (options & 1) )  { // is not a child
+		if (options & 2)
+			e1 = expand_dummy_sum(e, true);
+		e1 = canonicalize_clifford(e1);
+	}
+	
+	if (is_a<clifford>(e1) && ex_to<clifford>(e1).get_representation_label() >= rl) {
+		if (is_a<diracone>(e1.op(0)))
+			return 1;
+		else 
+			throw(std::invalid_argument("remove_dirac_ONE(): expression is a non-scalar Clifford number!"));
+	} else if (is_a<add>(e1) || is_a<ncmul>(e1) || is_a<mul>(e1)  
+			   || is_a<matrix>(e1) || e1.info(info_flags::list)) {
+		if (options & 3) // is a child or was already expanded
+			return e1.map(fcn);
+		else
+			try {
+				return e1.map(fcn);
+			} catch (std::exception &p) {
+				need_reevaluation = true;
+			}
+	} else if (is_a<power>(e1)) {
+		if (options & 3) // is a child or was already expanded
+			return pow(remove_dirac_ONE(e1.op(0), rl, options | 1), e1.op(1));
+		else
+			try {
+				return pow(remove_dirac_ONE(e1.op(0), rl, options | 1), e1.op(1));
+			} catch (std::exception &p) {
+				need_reevaluation = true;
+			}
+	} 
+	if (need_reevaluation)
+		return remove_dirac_ONE(e, rl, options | 2);
+	return e1;
 }
 
-ex remove_dirac_ONE(const ex & e,  unsigned char rl)
+char clifford_max_label(const ex & e, bool ignore_ONE)
 {
-	pointer_to_map_function_1arg<unsigned char> fcn(remove_dirac_ONE, rl);
-	if (is_a<clifford>(e) && ex_to<clifford>(e).get_representation_label() >= rl) {
-		if (is_a<diracone>(e.op(0)))
-			return 1;
+	if (is_a<clifford>(e))
+		if (ignore_ONE && is_a<diracone>(e.op(0)))
+			return -1;
 		else
-			throw(std::invalid_argument("Expression is a non-scalar Clifford number!"));
-	} else if (is_a<add>(e) || is_a<ncmul>(e) || is_a<mul>(e)  // || is_a<pseries>(e) || is_a<integral>(e)
-			   || is_a<matrix>(e) || is_a<lst>(e)) {
-		return e.map(fcn);
-	} else if (is_a<power>(e)) {
-		return pow(remove_dirac_ONE(e.op(0)), e.op(1));
-	} else
-		return e;
+			return ex_to<clifford>(e).get_representation_label();
+	else {
+		char rl = -1;
+		for (size_t i=0; i < e.nops(); i++) 
+			rl = (rl > clifford_max_label(e.op(i), ignore_ONE)) ? rl : clifford_max_label(e.op(i), ignore_ONE);
+		return rl;
+	}
 }
 
 ex clifford_norm(const ex & e)
 {
-	return sqrt(remove_dirac_ONE(canonicalize_clifford(e * clifford_bar(e)).simplify_indexed()));
+	return sqrt(remove_dirac_ONE(e * clifford_bar(e)));
 }
-
+	
 ex clifford_inverse(const ex & e)
 {
 	ex norm = clifford_norm(e);
 	if (!norm.is_zero())
 		return clifford_bar(e) / pow(norm, 2);
 	else 
-		throw(std::invalid_argument("Cannot find inverse of Clifford number with zero norm!"));
+		throw(std::invalid_argument("clifford_inverse(): cannot find inverse of Clifford number with zero norm!"));
 }
 
 ex lst_to_clifford(const ex & v, const ex & mu, const ex & metr, unsigned char rl)
 {
 	if (!ex_to<idx>(mu).is_dim_numeric())
-		throw(std::invalid_argument("Index should have a numeric dimension"));
+		throw(std::invalid_argument("lst_to_clifford(): Index should have a numeric dimension"));
 	ex e = clifford_unit(mu, metr, rl);
 	return lst_to_clifford(v, e);
 }
@@ -1064,8 +1167,10 @@ ex lst_to_clifford(const ex & v, const ex & e) {
 	unsigned min, max;
 
 	if (is_a<clifford>(e)) {
-		varidx mu = ex_to<varidx>(e.op(1));
-		unsigned dim = (ex_to<numeric>(mu.get_dim())).to_int();
+		ex mu = e.op(1);
+		ex mu_toggle
+			= is_a<varidx>(mu) ? ex_to<varidx>(mu).toggle_variance() : mu;
+		unsigned dim = (ex_to<numeric>(ex_to<idx>(mu).get_dim())).to_int();
 
 		if (is_a<matrix>(v)) {
 			if (ex_to<matrix>(v).cols() > ex_to<matrix>(v).rows()) {
@@ -1077,20 +1182,27 @@ ex lst_to_clifford(const ex & v, const ex & e) {
 			}
 			if (min == 1) {
 				if (dim == max)
-					return indexed(v, ex_to<varidx>(mu).toggle_variance()) * e;
-				else
-					throw(std::invalid_argument("Dimensions of vector and clifford unit mismatch"));
+					return indexed(v, mu_toggle) * e;
+				else if (max - dim == 1) {
+					if (ex_to<matrix>(v).cols() > ex_to<matrix>(v).rows())
+						return v.op(0) * dirac_ONE(ex_to<clifford>(e).get_representation_label()) + indexed(sub_matrix(ex_to<matrix>(v), 0, 1, 1, dim), mu_toggle) * e;
+					else 
+						return v.op(0) * dirac_ONE(ex_to<clifford>(e).get_representation_label()) + indexed(sub_matrix(ex_to<matrix>(v), 1, dim, 0, 1), mu_toggle) * e;
+ 				} else
+					throw(std::invalid_argument("lst_to_clifford(): dimensions of vector and clifford unit mismatch"));
 			} else
-				throw(std::invalid_argument("First argument should be a vector vector"));
-		} else if (is_a<lst>(v)) {
+				throw(std::invalid_argument("lst_to_clifford(): first argument should be a vector (nx1 or 1xn matrix)"));
+		} else if (v.info(info_flags::list)) {
 			if (dim == ex_to<lst>(v).nops())
-				return indexed(matrix(dim, 1, ex_to<lst>(v)), ex_to<varidx>(mu).toggle_variance()) * e;
+				return indexed(matrix(dim, 1, ex_to<lst>(v)), mu_toggle) * e;
+			else if (ex_to<lst>(v).nops() - dim == 1)
+				return v.op(0) * dirac_ONE(ex_to<clifford>(e).get_representation_label()) + indexed(sub_matrix(matrix(dim+1, 1, ex_to<lst>(v)), 1, dim, 0, 1), mu_toggle) * e;
 			else
-				throw(std::invalid_argument("List length and dimension of clifford unit mismatch"));
+				throw(std::invalid_argument("lst_to_clifford(): list length and dimension of clifford unit mismatch"));
 		} else
-			throw(std::invalid_argument("Cannot construct from anything but list or vector"));
+			throw(std::invalid_argument("lst_to_clifford(): cannot construct from anything but list or vector"));
 	} else
-		throw(std::invalid_argument("The second argument should be a Clifford unit"));
+		throw(std::invalid_argument("lst_to_clifford(): the second argument should be a Clifford unit"));
 }
  
 /** Auxiliary structure to define a function for striping one Clifford unit
@@ -1098,9 +1210,9 @@ ex lst_to_clifford(const ex & v, const ex & e) {
 static ex get_clifford_comp(const ex & e, const ex & c) 
 {
 	pointer_to_map_function_1arg<const ex &> fcn(get_clifford_comp, c);
-	int ival = ex_to<numeric>(ex_to<varidx>(c.op(1)).get_value()).to_int();
+	int ival = ex_to<numeric>(ex_to<idx>(c.op(1)).get_value()).to_int();
 		
-	if (is_a<add>(e) || is_a<lst>(e) // || is_a<pseries>(e) || is_a<integral>(e)
+	if (is_a<add>(e) || e.info(info_flags::list) // || is_a<pseries>(e) || is_a<integral>(e)
 		|| is_a<matrix>(e)) 
 		return e.map(fcn);
 	else if (is_a<ncmul>(e) || is_a<mul>(e)) {
@@ -1111,12 +1223,12 @@ static ex get_clifford_comp(const ex & e, const ex & c)
 				if (ind > e.nops()) 
 					ind = j;
 				else 
-					throw(std::invalid_argument("Expression is a Clifford multi-vector"));
+					throw(std::invalid_argument("get_clifford_comp(): expression is a Clifford multi-vector"));
 		if (ind < e.nops()) {
 			ex S = 1;
 			bool same_value_index, found_dummy;
-			same_value_index = ( ex_to<varidx>(e.op(ind).op(1)).is_numeric()
-								 &&  (ival == ex_to<numeric>(ex_to<varidx>(e.op(ind).op(1)).get_value()).to_int()) );
+			same_value_index = ( ex_to<idx>(e.op(ind).op(1)).is_numeric()
+								 &&  (ival == ex_to<numeric>(ex_to<idx>(e.op(ind).op(1)).get_value()).to_int()) );
 			found_dummy = same_value_index;
 			for(size_t j=0; j < e.nops(); j++)
 				if (j != ind) 
@@ -1128,7 +1240,12 @@ static ex get_clifford_comp(const ex & e, const ex & c)
 							found_dummy = true;
 							exvector::const_iterator it = ind_vec.begin(), itend = ind_vec.end();
 							while (it != itend) {
-								S = S * e.op(j).subs(lst(ex_to<varidx>(*it) == ival, ex_to<varidx>(*it).toggle_variance() == ival), subs_options::no_pattern);
+								ex curridx = *it;
+								ex curridx_toggle = is_a<varidx>(curridx)
+									? ex_to<varidx>(curridx).toggle_variance()
+									: curridx;
+								S = S * e.op(j).subs(lst(curridx == ival,
+									curridx_toggle == ival), subs_options::no_pattern);
 								++it;
 							}
 						} else
@@ -1136,43 +1253,59 @@ static ex get_clifford_comp(const ex & e, const ex & c)
 					}
 			return (found_dummy ? S : 0);
 		} else
-			throw(std::invalid_argument("Expression is not a Clifford vector to the given units"));
+			throw(std::invalid_argument("get_clifford_comp(): expression is not a Clifford vector to the given units"));
 	} else if (e.is_zero()) 
 		return e;
 	else if (is_a<clifford>(e) && ex_to<clifford>(e).same_metric(c))
-		if ( ex_to<varidx>(e.op(1)).is_numeric() &&
-			 (ival != ex_to<numeric>(ex_to<varidx>(e.op(1)).get_value()).to_int()) )
+		if ( ex_to<idx>(e.op(1)).is_numeric() &&
+			 (ival != ex_to<numeric>(ex_to<idx>(e.op(1)).get_value()).to_int()) )
 			return 0;
 		else 
 			return 1;
 	else
-		throw(std::invalid_argument("Expression is not usable as a Clifford vector"));
+		throw(std::invalid_argument("get_clifford_comp(): expression is not usable as a Clifford vector"));
 }
 
 
 lst clifford_to_lst(const ex & e, const ex & c, bool algebraic)
 {
 	GINAC_ASSERT(is_a<clifford>(c));
-	varidx mu = ex_to<varidx>(c.op(1));
-	if (! mu.is_dim_numeric())
-		throw(std::invalid_argument("Index should have a numeric dimension"));
-	unsigned int D = ex_to<numeric>(mu.get_dim()).to_int();
+	ex mu = c.op(1);
+	if (! ex_to<idx>(mu).is_dim_numeric())
+		throw(std::invalid_argument("clifford_to_lst(): index should have a numeric dimension"));
+	unsigned int D = ex_to<numeric>(ex_to<idx>(mu).get_dim()).to_int();
 
 	if (algebraic) // check if algebraic method is applicable
 		for (unsigned int i = 0; i < D; i++) 
-			if (pow(c.subs(mu == i), 2).is_zero() 
-				or (not is_a<numeric>(pow(c.subs(mu == i), 2))))
+			if (pow(c.subs(mu == i, subs_options::no_pattern), 2).is_zero() 
+				or (not is_a<numeric>(pow(c.subs(mu == i, subs_options::no_pattern), 2))))
 				algebraic = false;
 	lst V; 
-	if (algebraic) 
+	ex v0 = remove_dirac_ONE(canonicalize_clifford(e+clifford_prime(e)).normal())/2;
+	if (not v0.is_zero())
+		V.append(v0);
+	ex e1 = canonicalize_clifford(e - v0 * dirac_ONE(ex_to<clifford>(c).get_representation_label())); 
+	if (algebraic) {
 		for (unsigned int i = 0; i < D; i++) 
 			V.append(remove_dirac_ONE(
-						simplify_indexed(canonicalize_clifford(e * c.subs(mu == i) +  c.subs(mu == i) * e))
-						/ (2*pow(c.subs(mu == i), 2))));
-	else {
-		ex e1 = canonicalize_clifford(e);
-		for (unsigned int i = 0; i < D; i++) 
-			V.append(get_clifford_comp(e1, c.subs(c.op(1) == i)));
+						simplify_indexed(canonicalize_clifford(e1 * c.subs(mu == i, subs_options::no_pattern) +  c.subs(mu == i, subs_options::no_pattern) * e1))
+						/ (2*pow(c.subs(mu == i, subs_options::no_pattern), 2))));
+	} else {
+		try {
+			for (unsigned int i = 0; i < D; i++) 
+				V.append(get_clifford_comp(e1, c.subs(c.op(1) == i, subs_options::no_pattern)));
+		} catch  (std::exception &p) {
+			/* Try to expand dummy summations to simplify the expression*/
+			e1 = canonicalize_clifford(expand_dummy_sum(e, true));
+			V.remove_all();
+			v0 = remove_dirac_ONE(canonicalize_clifford(e1+clifford_prime(e1)).normal())/2;
+			if (not v0.is_zero()) {
+				V.append(v0);
+				e1 = canonicalize_clifford(e1 - v0 * dirac_ONE(ex_to<clifford>(c).get_representation_label())); 
+			}
+			for (unsigned int i = 0; i < D; i++) 
+				V.append(get_clifford_comp(e1, c.subs(c.op(1) == i, subs_options::no_pattern)));
+		}
 	}
 	return V;
 }
@@ -1182,44 +1315,35 @@ ex clifford_moebius_map(const ex & a, const ex & b, const ex & c, const ex & d,
 {
 	ex x, D, cu;
 	
-	if (! is_a<matrix>(v) && ! is_a<lst>(v))
-		throw(std::invalid_argument("parameter v should be either vector or list"));
+	if (! is_a<matrix>(v) && ! v.info(info_flags::list))
+		throw(std::invalid_argument("clifford_moebius_map(): parameter v should be either vector or list"));
 	
 	if (is_a<clifford>(G)) {
 		cu = G;
 	} else {
-		if (is_a<indexed>(G)) 
-			D = ex_to<varidx>(G.op(1)).get_dim();
-		else if (is_a<matrix>(G)) 
+		if (is_a<indexed>(G)) {
+			D = ex_to<idx>(G.op(1)).get_dim();
+			varidx mu((new symbol)->setflag(status_flags::dynallocated), D);
+			cu = clifford_unit(mu, G, rl);
+		} else if (is_a<matrix>(G)) {
 			D = ex_to<matrix>(G).rows(); 
-		else throw(std::invalid_argument("metric should be an indexed object, matrix, or a Clifford unit"));
+			idx mu((new symbol)->setflag(status_flags::dynallocated), D);
+			cu = clifford_unit(mu, G, rl);
+		} else throw(std::invalid_argument("clifford_moebius_map(): metric should be an indexed object, matrix, or a Clifford unit"));
 		
-		varidx mu((new symbol)->setflag(status_flags::dynallocated), D);
-		cu = clifford_unit(mu, G, rl);
 	}
-
+	
 	x = lst_to_clifford(v, cu); 
-	ex e = simplify_indexed(canonicalize_clifford((a * x + b) * clifford_inverse(c * x + d)));
-	return clifford_to_lst(e, cu, false);
+	ex e = clifford_to_lst(simplify_indexed(canonicalize_clifford((a * x + b) * clifford_inverse(c * x + d))), cu, false);
+	return (is_a<matrix>(v) ? matrix(ex_to<matrix>(v).rows(), ex_to<matrix>(v).cols(), ex_to<lst>(e)) : e);
 }
 
 ex clifford_moebius_map(const ex & M, const ex & v, const ex & G, unsigned char rl)
 {
-	if (is_a<matrix>(M)) 
-		return clifford_moebius_map(ex_to<matrix>(M)(0,0), ex_to<matrix>(M)(0,1), 
-						ex_to<matrix>(M)(1,0), ex_to<matrix>(M)(1,1), v, G, rl);
+	if (is_a<matrix>(M) && (ex_to<matrix>(M).rows() == 2) && (ex_to<matrix>(M).cols() == 2)) 
+		return clifford_moebius_map(M.op(0), M.op(1), M.op(2), M.op(3), v, G, rl);
 	else
-		throw(std::invalid_argument("parameter M should be a matrix"));
-}
-
-ex clifford_moebius_map(const ex & M, const ex & v, const ex & G)
-{
-	return clifford_moebius_map(M, v, G, 0);
-}
-
-ex clifford_moebius_map(const ex & a, const ex & b, const ex & c, const ex & d, const ex & v, const ex & G)
-{
-	return clifford_moebius_map(a, b, c, d, v, G, 0);
+		throw(std::invalid_argument("clifford_moebius_map(): parameter M should be a 2x2 matrix"));
 }
 
 } // namespace GiNaC
