@@ -39,6 +39,8 @@ namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS(color, indexed)
 
+const tinfo_static_t color::return_type_tinfo_static[256] = {{}};
+
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(su3one, tensor,
   print_func<print_dflt>(&su3one::do_print).
   print_func<print_latex>(&su3one::do_print_latex))
@@ -61,7 +63,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(su3d, tensor,
 
 color::color() : representation_label(0)
 {
-	tinfo_key = TINFO_color;
+	tinfo_key = &color::tinfo_static;
 }
 
 DEFAULT_CTOR(su3one)
@@ -78,7 +80,7 @@ DEFAULT_CTOR(su3d)
  *  @see color_ONE */
 color::color(const ex & b, unsigned char rl) : inherited(b), representation_label(rl)
 {
-	tinfo_key = TINFO_color;
+	tinfo_key = &color::tinfo_static;
 }
 
 /** Construct object with one color index. This constructor is for internal
@@ -86,17 +88,17 @@ color::color(const ex & b, unsigned char rl) : inherited(b), representation_labe
  *  @see color_T */
 color::color(const ex & b, const ex & i1, unsigned char rl) : inherited(b, i1), representation_label(rl)
 {
-	tinfo_key = TINFO_color;
+	tinfo_key = &color::tinfo_static;
 }
 
 color::color(unsigned char rl, const exvector & v, bool discardable) : inherited(not_symmetric(), v, discardable), representation_label(rl)
 {
-	tinfo_key = TINFO_color;
+	tinfo_key = &color::tinfo_static;
 }
 
 color::color(unsigned char rl, std::auto_ptr<exvector> vp) : inherited(not_symmetric(), vp), representation_label(rl)
 {
-	tinfo_key = TINFO_color;
+	tinfo_key = &color::tinfo_static;
 }
 
 //////////
@@ -520,25 +522,19 @@ ex color_h(const ex & a, const ex & b, const ex & c)
 	return color_d(a, b, c) + I * color_f(a, b, c);
 }
 
-/** Check whether a given tinfo key (as returned by return_type_tinfo()
- *  is that of a color object with the specified representation label. */
-static bool is_color_tinfo(unsigned ti, unsigned char rl)
-{
-	return ti == (TINFO_color + rl);
-}
-
 /** Check whether a given tinfo key (as returned by return_type_tinfo()
  *  is that of a color object (with an arbitrary representation label). */
-static bool is_color_tinfo(unsigned ti)
+static bool is_color_tinfo(tinfo_t ti)
 {
-	return (ti & ~0xff) == TINFO_color;
+	p_int start_loc=(p_int)&color::return_type_tinfo_static;
+	return (p_int)ti>=start_loc && (p_int)ti<start_loc+256;
 }
 
 /** Extract representation label from tinfo key (as returned by
  *  return_type_tinfo()). */
-static unsigned char get_representation_label(unsigned ti)
+static unsigned char get_representation_label(tinfo_t ti)
 {
-	return ti & 0xff;
+	return (unsigned char)((p_int)ti-(p_int)&color::return_type_tinfo_static);
 }
 
 ex color_trace(const ex & e, const std::set<unsigned char> & rls)
