@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's constant types and some special constants. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "ex.h"
 #include "archive.h"
 #include "utils.h"
+#include "inifcns.h"
 
 namespace GiNaC {
 
@@ -44,7 +45,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(constant, basic,
 
 // public
 
-constant::constant() : basic(TINFO_constant), ef(0), serial(next_serial++)
+constant::constant() : basic(&constant::tinfo_static), ef(0), serial(next_serial++), domain(domain::complex)
 {
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
@@ -55,8 +56,8 @@ constant::constant() : basic(TINFO_constant), ef(0), serial(next_serial++)
 
 // public
 
-constant::constant(const std::string & initname, evalffunctype efun, const std::string & texname)
-  : basic(TINFO_constant), name(initname), ef(efun), serial(next_serial++)
+constant::constant(const std::string & initname, evalffunctype efun, const std::string & texname, unsigned dm)
+  : basic(&constant::tinfo_static), name(initname), ef(efun), serial(next_serial++), domain(dm)
 {
 	if (texname.empty())
 		TeX_name = "\\mbox{" + name + "}";
@@ -65,8 +66,8 @@ constant::constant(const std::string & initname, evalffunctype efun, const std::
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
 
-constant::constant(const std::string & initname, const numeric & initnumber, const std::string & texname)
-  : basic(TINFO_constant), name(initname), ef(0), number(initnumber), serial(next_serial++)
+constant::constant(const std::string & initname, const numeric & initnumber, const std::string & texname, unsigned dm)
+  : basic(&constant::tinfo_static), name(initname), ef(0), number(initnumber), serial(next_serial++), domain(dm)
 {
 	if (texname.empty())
 		TeX_name = "\\mbox{" + name + "}";
@@ -136,6 +137,18 @@ void constant::do_print_python_repr(const print_python_repr & c, unsigned level)
 	c.s << ')';
 }
 
+bool constant::info(unsigned inf) const
+{
+	if (inf == info_flags::polynomial)
+		return true;
+	if (inf == info_flags::real)
+		return domain==domain::real || domain==domain::positive ;
+	if (inf==info_flags::positive || inf==info_flags::nonnegative)
+		return domain == domain::positive;
+	else
+		return inherited::info(inf);
+}
+
 ex constant::evalf(int level) const
 {
 	if (ef!=0) {
@@ -146,6 +159,32 @@ ex constant::evalf(int level) const
 	return *this;
 }
 
+bool constant::is_polynomial(const ex & var) const
+{
+	return true;
+}
+
+ex constant::conjugate() const
+{
+	if ( domain==domain::real || domain==domain::positive )
+		return *this;
+	return conjugate_function(*this).hold();
+}
+
+ex constant::real_part() const
+{
+	if ( domain==domain::real || domain==domain::positive )
+		return *this;
+	return real_part_function(*this).hold();
+}
+
+ex constant::imag_part() const
+{
+	if ( domain==domain::real || domain==domain::positive )
+		return 0;
+	return imag_part_function(*this).hold();
+}
+
 // protected
 
 /** Implementation of ex::diff() for a constant always returns 0.
@@ -177,7 +216,7 @@ bool constant::is_equal_same_type(const basic & other) const
 
 unsigned constant::calchash() const
 {
-	hashvalue = golden_ratio_hash(tinfo() ^ serial);
+	hashvalue = golden_ratio_hash((p_int)tinfo() ^ serial);
 
 	setflag(status_flags::hash_calculated);
 
@@ -207,13 +246,13 @@ unsigned constant::next_serial = 0;
 //////////
 
 /**  Pi. (3.14159...)  Diverts straight into CLN for evalf(). */
-const constant Pi("Pi", PiEvalf, "\\pi");
+const constant Pi("Pi", PiEvalf, "\\pi", domain::positive);
 
 /** Euler's constant. (0.57721...)  Sometimes called Euler-Mascheroni constant.
  *  Diverts straight into CLN for evalf(). */
-const constant Euler("Euler", EulerEvalf, "\\gamma_E");
+const constant Euler("Euler", EulerEvalf, "\\gamma_E", domain::positive);
 
 /** Catalan's constant. (0.91597...)  Diverts straight into CLN for evalf(). */
-const constant Catalan("Catalan", CatalanEvalf, "G");
+const constant Catalan("Catalan", CatalanEvalf, "G", domain::positive);
 
 } // namespace GiNaC
