@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's light-weight expression handles. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "mul.h"
 #include "ncmul.h"
 #include "numeric.h"
+#include "matrix.h"
 #include "power.h"
 #include "lst.h"
 #include "relational.h"
@@ -240,6 +241,31 @@ ex ex::rhs() const
 	return bp->op(1);
 }
 
+/** Check whether expression is a polynomial. */
+bool ex::is_polynomial(const ex & vars) const
+{
+	if (is_a<lst>(vars)) {
+		const lst & varlst = ex_to<lst>(vars);
+		for (lst::const_iterator i=varlst.begin(); i!=varlst.end(); ++i)
+			if (!bp->is_polynomial(*i))
+				return false;
+		return true;
+	}
+	else
+		return bp->is_polynomial(vars);
+}
+
+/** Check whether expression is zero or zero matrix. */
+bool ex::is_zero_matrix() const
+{
+	if (is_zero())
+		return  true;
+	else {
+		ex e = evalm();
+		return is_a<matrix>(e) && ex_to<matrix>(e).is_zero_matrix();
+	}
+}
+
 // private
 
 /** Make this ex writable (if more than one ex handle the same basic) by 
