@@ -52,6 +52,12 @@ private:
 /** For construction of flyweights, etc. */
 static library_init library_initializer;
 
+/** Rotate bits of unsigned value by one bit to the left.
+  * This can be necesary if the user wants to define its own hashes. */
+inline unsigned rotate_left(unsigned n)
+{
+	return (n & 0x80000000U) ? (n << 1 | 0x00000001U) : (n << 1);
+}
 
 class scalar_products;
 class const_iterator;
@@ -74,10 +80,6 @@ class ex {
 	// default constructor, copy constructor and assignment operator
 public:
 	ex() throw();
-#ifdef OBSCURE_CINT_HACK
-	ex(const ex & other);
-	ex & operator=(const ex & other);
-#endif
 
 	// other constructors
 public:
@@ -139,11 +141,13 @@ public:
 	ex lhs() const;
 	ex rhs() const;
 
-	// complex conjugation
+	// function for complex expressions
 	ex conjugate() const { return bp->conjugate(); }
+	ex real_part() const { return bp->real_part(); }
+	ex imag_part() const { return bp->imag_part(); }
 
 	// pattern matching
-	bool has(const ex & pattern) const { return bp->has(pattern); }
+	bool has(const ex & pattern, unsigned options = 0) const { return bp->has(pattern, options); }
 	bool find(const ex & pattern, lst & found) const;
 	bool match(const ex & pattern) const;
 	bool match(const ex & pattern, lst & repl_lst) const { return bp->match(pattern, repl_lst); }
@@ -164,6 +168,7 @@ public:
 	void traverse(visitor & v) const { traverse_preorder(v); }
 
 	// degree/coeff
+	bool is_polynomial(const ex & vars) const;
 	int degree(const ex & s) const { return bp->degree(s); }
 	int ldegree(const ex & s) const { return bp->ldegree(s); }
 	ex coeff(const ex & s, int n = 1) const { return bp->coeff(s, n); }
@@ -207,6 +212,7 @@ public:
 	int compare(const ex & other) const;
 	bool is_equal(const ex & other) const;
 	bool is_zero() const { extern const ex _ex0; return is_equal(_ex0); }
+	bool is_zero_matrix() const;
 	
 	// symmetry
 	ex symmetrize() const;
@@ -218,7 +224,7 @@ public:
 
 	// noncommutativity
 	unsigned return_type() const { return bp->return_type(); }
-	unsigned return_type_tinfo() const { return bp->return_type_tinfo(); }
+	tinfo_t return_type_tinfo() const { return bp->return_type_tinfo(); }
 
 	unsigned gethash() const { return bp->gethash(); }
 
@@ -233,36 +239,10 @@ private:
 	void makewriteable();
 	void share(const ex & other) const;
 
-#ifdef OBSCURE_CINT_HACK
-public:
-	static bool last_created_or_assigned_bp_can_be_converted_to_ex()
-	{
-		if (last_created_or_assigned_bp==0) return false;
-		if ((last_created_or_assigned_bp->flags &
-			 status_flags::dynallocated)==0) return false;
-		if ((last_created_or_assigned_bp->flags &
-			 status_flags::evaluated)==0) return false;
-		return true;
-	}
-protected:
-	void update_last_created_or_assigned_bp()
-	{
-		last_created_or_assigned_bp = bp;
-		last_created_or_assigned_exp = (long)(void *)(this);
-	}
-#endif // def OBSCURE_CINT_HACK
-
 // member variables
 
 private:
 	mutable ptr<basic> bp;  ///< pointer to basic object managed by this
-
-#ifdef OBSCURE_CINT_HACK
-public:
-	static ptr<basic> last_created_or_assigned_bp;
-	static basic * dummy_bp;
-	static long last_created_or_assigned_exp;
-#endif // def OBSCURE_CINT_HACK
 };
 
 
@@ -276,91 +256,48 @@ inline
 ex::ex() throw() : bp(*const_cast<basic *>(_num0_bp))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-#ifdef OBSCURE_CINT_HACK
-	update_last_created_or_assigned_bp();
-#endif // def OBSCURE_CINT_HACK
 }
 
-#ifdef OBSCURE_CINT_HACK
-inline
-ex::ex(const ex & other) : bp(other.bp)
-{
-	GINAC_ASSERT((bp->flags) & status_flags::dynallocated);
-	update_last_created_or_assigned_bp();
-}
-
-inline
-ex & ex::operator=(const ex & other)
-{
-	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-	GINAC_ASSERT(other.bp->flags & status_flags::dynallocated);
-	bp = other.bp;
-	update_last_created_or_assigned_bp();
-	return *this;
-}
-#endif // def OBSCURE_CINT_HACK
-
 inline
 ex::ex(const basic & other) : bp(construct_from_basic(other))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-#ifdef OBSCURE_CINT_HACK
-	update_last_created_or_assigned_bp();
-#endif // def OBSCURE_CINT_HACK
 }
 
 inline
 ex::ex(int i) : bp(construct_from_int(i))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-#ifdef OBSCURE_CINT_HACK
-	update_last_created_or_assigned_bp();
-#endif // def OBSCURE_CINT_HACK
 }
 
 inline
 ex::ex(unsigned int i) : bp(construct_from_uint(i))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-#ifdef OBSCURE_CINT_HACK
-	update_last_created_or_assigned_bp();
-#endif // def OBSCURE_CINT_HACK
 }
 
 inline
 ex::ex(long i) : bp(construct_from_long(i))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-#ifdef OBSCURE_CINT_HACK
-	update_last_created_or_assigned_bp();
-#endif // def OBSCURE_CINT_HACK
 }
 
 inline
 ex::ex(unsigned long i) : bp(construct_from_ulong(i))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-#ifdef OBSCURE_CINT_HACK
-	update_last_created_or_assigned_bp();
-#endif // def OBSCURE_CINT_HACK
 }
 
 inline
 ex::ex(double const d) : bp(construct_from_double(d))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-#ifdef OBSCURE_CINT_HACK
-	update_last_created_or_assigned_bp();
-#endif // def OBSCURE_CINT_HACK
 }
 
 inline
 ex::ex(const std::string &s, const ex &l) : bp(construct_from_string_and_lst(s, l))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-#ifdef OBSCURE_CINT_HACK
-	update_last_created_or_assigned_bp();
-#endif // def OBSCURE_CINT_HACK
 }
 
 inline
@@ -397,7 +334,7 @@ bool ex::is_equal(const ex & other) const
 #ifdef GINAC_COMPARE_STATISTICS
 	compare_statistics.nontrivial_is_equals++;
 #endif
-    const bool equal = bp->is_equal(*other.bp);
+	const bool equal = bp->is_equal(*other.bp);
 #if 0
 	if (equal) {
 		// Expressions point to different, but equal, trees: conserve
@@ -748,6 +685,11 @@ struct ex_swap : public std::binary_function<ex, ex, void> {
 	void operator() (ex &lh, ex &rh) const { lh.swap(rh); }
 };
 
+// Make it possible to print exvectors and exmaps
+std::ostream & operator<<(std::ostream & os, const exvector & e);
+std::ostream & operator<<(std::ostream & os, const exset & e);
+std::ostream & operator<<(std::ostream & os, const exmap & e);
+
 // wrapper functions around member functions
 inline size_t nops(const ex & thisex)
 { return thisex.nops(); }
@@ -758,12 +700,21 @@ inline ex expand(const ex & thisex, unsigned options = 0)
 inline ex conjugate(const ex & thisex)
 { return thisex.conjugate(); }
 
-inline bool has(const ex & thisex, const ex & pattern)
-{ return thisex.has(pattern); }
+inline ex real_part(const ex & thisex)
+{ return thisex.real_part(); }
+
+inline ex imag_part(const ex & thisex)
+{ return thisex.imag_part(); }
+
+inline bool has(const ex & thisex, const ex & pattern, unsigned options = 0)
+{ return thisex.has(pattern, options); }
 
 inline bool find(const ex & thisex, const ex & pattern, lst & found)
 { return thisex.find(pattern, found); }
 
+inline bool is_polynomial(const ex & thisex, const ex & vars)
+{ return thisex.is_polynomial(vars); }
+
 inline int degree(const ex & thisex, const ex & s)
 { return thisex.degree(s); }
 
@@ -917,6 +868,52 @@ public:
 	ex operator()(const ex & e) { return ptr(e, arg1, arg2, arg3); }
 };
 
+template<class C>
+class pointer_to_member_to_map_function : public map_function {
+protected:
+	ex (C::*ptr)(const ex &);
+	C &c;
+public:
+	explicit pointer_to_member_to_map_function(ex (C::*member)(const ex &), C &obj) : ptr(member), c(obj) {}
+	ex operator()(const ex & e) { return (c.*ptr)(e); }
+};
+
+template<class C, class T1>
+class pointer_to_member_to_map_function_1arg : public map_function {
+protected:
+	ex (C::*ptr)(const ex &, T1);
+	C &c;
+	T1 arg1;
+public:
+	explicit pointer_to_member_to_map_function_1arg(ex (C::*member)(const ex &, T1), C &obj, T1 a1) : ptr(member), c(obj), arg1(a1) {}
+	ex operator()(const ex & e) { return (c.*ptr)(e, arg1); }
+};
+
+template<class C, class T1, class T2>
+class pointer_to_member_to_map_function_2args : public map_function {
+protected:
+	ex (C::*ptr)(const ex &, T1, T2);
+	C &c;
+	T1 arg1;
+	T2 arg2;
+public:
+	explicit pointer_to_member_to_map_function_2args(ex (C::*member)(const ex&, T1, T2), C &obj, T1 a1, T2 a2) : ptr(member), c(obj), arg1(a1), arg2(a2) {}
+	ex operator()(const ex & e) { return (c.*ptr)(e, arg1, arg2); }
+};
+
+template<class C, class T1, class T2, class T3>
+class pointer_to_member_to_map_function_3args : public map_function {
+protected:
+	ex (C::*ptr)(const ex &, T1, T2, T3);
+	C &c;
+	T1 arg1;
+	T2 arg2;
+	T3 arg3;
+public:
+	explicit pointer_to_member_to_map_function_3args(ex (C::*member)(const ex &, T1, T2, T3), C &obj, T1 a1, T2 a2, T3 a3) : ptr(member), c(obj), arg1(a1), arg2(a2), arg3(a3) {}
+	ex operator()(const ex & e) { return (c.*ptr)(e, arg1, arg2, arg3); }
+};
+
 inline ex ex::map(ex f(const ex &)) const
 {
 	pointer_to_map_function fcn(f);
