@@ -32,17 +32,11 @@ namespace GiNaC {
 typedef container<std::vector> exprseq;
 
 /** Specialization of container::get_tinfo() for exprseq. */
-template<> inline unsigned exprseq::get_tinfo() { return TINFO_exprseq; }
+template<> inline tinfo_t exprseq::get_tinfo() { return &exprseq::tinfo_static; }
 
 // defined in exprseq.cpp
 template<> bool exprseq::info(unsigned inf) const;
 
-/** Specialization of is_exactly_a<exprseq>(obj) for exprseq objects. */
-template<> inline bool is_exactly_a<exprseq>(const basic & obj)
-{
-	return obj.tinfo() == TINFO_exprseq;
-}
-
 } // namespace GiNaC
 
 #endif // ndef __GINAC_EXPRSEQ_H__
