@@ -31,6 +31,7 @@ namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(fderivative, function,
   print_func<print_context>(&fderivative::do_print).
+  print_func<print_csrc>(&fderivative::do_print_csrc).
   print_func<print_tree>(&fderivative::do_print_tree))
 
 //////////
@@ -39,7 +40,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(fderivative, function,
 
 fderivative::fderivative()
 {
-	tinfo_key = TINFO_fderivative;
+	tinfo_key = &fderivative::tinfo_static;
 }
 
 //////////
@@ -49,17 +50,17 @@ fderivative::fderivative()
 fderivative::fderivative(unsigned ser, unsigned param, const exvector & args) : function(ser, args)
 {
 	parameter_set.insert(param);
-	tinfo_key = TINFO_fderivative;
+	tinfo_key = &fderivative::tinfo_static;
 }
 
 fderivative::fderivative(unsigned ser, const paramset & params, const exvector & args) : function(ser, args), parameter_set(params)
 {
-	tinfo_key = TINFO_fderivative;
+	tinfo_key = &fderivative::tinfo_static;
 }
 
 fderivative::fderivative(unsigned ser, const paramset & params, std::auto_ptr<exvector> vp) : function(ser, vp), parameter_set(params)
 {
-	tinfo_key = TINFO_fderivative;
+	tinfo_key = &fderivative::tinfo_static;
 }
 
 //////////
@@ -106,12 +107,24 @@ void fderivative::do_print(const print_context & c, unsigned level) const
 	c.s << "D[";
 	paramset::const_iterator i = parameter_set.begin(), end = parameter_set.end();
 	--end;
-	while (i != end)
+	while (i != end) {
 		c.s << *i++ << ",";
+	}
 	c.s << *i << "](" << registered_functions()[serial].name << ")";
 	printseq(c, '(', ',', ')', exprseq::precedence(), function::precedence());
 }
 
+void fderivative::do_print_csrc(const print_csrc & c, unsigned level) const
+{
+	c.s << "D_";
+	paramset::const_iterator i = parameter_set.begin(), end = parameter_set.end();
+	--end;
+	while (i != end)
+		c.s << *i++ << "_";
+	c.s << *i << "_" << registered_functions()[serial].name;
+	printseq(c, '(', ',', ')', exprseq::precedence(), function::precedence());
+}
+
 void fderivative::do_print_tree(const print_tree & c, unsigned level) const
 {
 	c.s << std::string(level, ' ') << class_name() << " "
