@@ -3,7 +3,7 @@
  *  Interface to abstract derivatives of functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef __GINAC_FDERIVATIVE_H__
@@ -74,6 +74,7 @@ protected:
 	// non-virtual functions in this class
 protected:
 	void do_print(const print_context & c, unsigned level) const;
+	void do_print_csrc(const print_csrc & c, unsigned level) const;
 	void do_print_tree(const print_tree & c, unsigned level) const;
 
 	// member variables
@@ -81,14 +82,6 @@ protected:
 	paramset parameter_set; /**< Set of parameter numbers with respect to which to take the derivative */
 };
 
-// utility functions
-
-/** Specialization of is_exactly_a<T>(obj) for derivatives. */
-template<> inline bool is_exactly_a<fderivative>(const basic & obj)
-{
-	return obj.tinfo()==TINFO_fderivative;
-}
-
 } // namespace GiNaC
 
 #endif // ndef __GINAC_DERIVATIVE_H__
