@@ -37,6 +37,7 @@ namespace GiNaC {
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(idx, basic,
   print_func<print_context>(&idx::do_print).
   print_func<print_latex>(&idx::do_print_latex).
+  print_func<print_csrc>(&idx::do_print_csrc).
   print_func<print_tree>(&idx::do_print_tree))
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(varidx, idx,
@@ -53,23 +54,23 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(spinidx, varidx,
 // default constructor
 //////////
 
-idx::idx() : inherited(TINFO_idx) {}
+idx::idx() : inherited(&idx::tinfo_static) {}
 
 varidx::varidx() : covariant(false)
 {
-	tinfo_key = TINFO_varidx;
+	tinfo_key = &varidx::tinfo_static;
 }
 
 spinidx::spinidx() : dotted(false)
 {
-	tinfo_key = TINFO_spinidx;
+	tinfo_key = &spinidx::tinfo_static;
 }
 
 //////////
 // other constructors
 //////////
 
-idx::idx(const ex & v, const ex & d) : inherited(TINFO_idx), value(v), dim(d)
+idx::idx(const ex & v, const ex & d) : inherited(&idx::tinfo_static), value(v), dim(d)
 {
 	if (is_dim_numeric())
 		if (!dim.info(info_flags::posint))
@@ -78,12 +79,12 @@ idx::idx(const ex & v, const ex & d) : inherited(TINFO_idx), value(v), dim(d)
 
 varidx::varidx(const ex & v, const ex & d, bool cov) : inherited(v, d), covariant(cov)
 {
-	tinfo_key = TINFO_varidx;
+	tinfo_key = &varidx::tinfo_static;
 }
 
 spinidx::spinidx(const ex & v, const ex & d, bool cov, bool dot) : inherited(v, d, cov), dotted(dot)
 {
-	tinfo_key = TINFO_spinidx;
+	tinfo_key = &spinidx::tinfo_static;
 }
 
 //////////
@@ -161,6 +162,16 @@ void idx::do_print_latex(const print_latex & c, unsigned level) const
 	c.s << "}";
 }
 
+void idx::do_print_csrc(const print_csrc & c, unsigned level) const
+{
+	c.s << "[";
+	if (value.info(info_flags::integer))
+		c.s << ex_to<numeric>(value).to_int();
+	else
+		value.print(c);
+	c.s << "]";
+}
+
 void idx::do_print_tree(const print_tree & c, unsigned level) const
 {
 	c.s << std::string(level, ' ') << class_name() << " @" << this
@@ -223,8 +234,11 @@ void spinidx::do_print_tree(const print_tree & c, unsigned level) const
 
 bool idx::info(unsigned inf) const
 {
-	if (inf == info_flags::idx)
-		return true;
+	switch(inf) {
+		case info_flags::idx:
+		case info_flags::has_indices:
+			return true;
+	}
 	return inherited::info(inf);
 }
 
@@ -338,7 +352,7 @@ unsigned idx::calchash() const
 	// hash keys. That is, the hash values must not depend on the index
 	// dimensions or other attributes (variance etc.).
 	// The compare_same_type() methods will take care of the rest.
-	unsigned v = golden_ratio_hash(tinfo());
+	unsigned v = golden_ratio_hash((p_int)tinfo());
 	v = rotate_left(v);
 	v ^= value.gethash();
 
@@ -365,7 +379,7 @@ ex idx::subs(const exmap & m, unsigned options) const
 	if (it != m.end()) {
 
 		// Substitution index->index
-		if (is_a<idx>(it->second))
+		if (is_a<idx>(it->second) || (options & subs_options::really_subs_idx))
 			return it->second;
 
 		// Otherwise substitute value
