@@ -23,6 +23,7 @@
 #include <iostream>
 #include <sstream>
 #include <stdexcept>
+#include <limits>
 
 #include "indexed.h"
 #include "idx.h"
@@ -39,6 +40,7 @@
 #include "utils.h"
 #include "integral.h"
 #include "matrix.h"
+#include "inifcns.h"
 
 namespace GiNaC {
 
@@ -53,7 +55,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(indexed, exprseq,
 
 indexed::indexed() : symtree(not_symmetric())
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 }
 
 //////////
@@ -62,79 +64,79 @@ indexed::indexed() : symtree(not_symmetric())
 
 indexed::indexed(const ex & b) : inherited(b), symtree(not_symmetric())
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const ex & i1) : inherited(b, i1), symtree(not_symmetric())
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const ex & i1, const ex & i2) : inherited(b, i1, i2), symtree(not_symmetric())
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3) : inherited(b, i1, i2, i3), symtree(not_symmetric())
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3, const ex & i4) : inherited(b, i1, i2, i3, i4), symtree(not_symmetric())
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2) : inherited(b, i1, i2), symtree(symm)
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2, const ex & i3) : inherited(b, i1, i2, i3), symtree(symm)
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex & i2, const ex & i3, const ex & i4) : inherited(b, i1, i2, i3, i4), symtree(symm)
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const exvector & v) : inherited(b), symtree(not_symmetric())
 {
 	seq.insert(seq.end(), v.begin(), v.end());
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const ex & b, const symmetry & symm, const exvector & v) : inherited(b), symtree(symm)
 {
 	seq.insert(seq.end(), v.begin(), v.end());
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 	validate();
 }
 
 indexed::indexed(const symmetry & symm, const exprseq & es) : inherited(es), symtree(symm)
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 }
 
 indexed::indexed(const symmetry & symm, const exvector & v, bool discardable) : inherited(v, discardable), symtree(symm)
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 }
 
 indexed::indexed(const symmetry & symm, std::auto_ptr<exvector> vp) : inherited(vp), symtree(symm)
 {
-	tinfo_key = TINFO_indexed;
+	tinfo_key = &indexed::tinfo_static;
 }
 
 //////////
@@ -297,7 +299,7 @@ ex indexed::eval(int level) const
 		return f * thiscontainer(v);
 	}
 
-	if(this->tinfo()==TINFO_indexed && seq.size()==1)
+	if(this->tinfo()==&indexed::tinfo_static && seq.size()==1)
 		return base;
 
 	// Canonicalize indices according to the symmetry properties
@@ -305,7 +307,7 @@ ex indexed::eval(int level) const
 		exvector v = seq;
 		GINAC_ASSERT(is_exactly_a<symmetry>(symtree));
 		int sig = canonicalize(v.begin() + 1, ex_to<symmetry>(symtree));
-		if (sig != INT_MAX) {
+		if (sig != std::numeric_limits<int>::max()) {
 			// Something has changed while sorting indices, more evaluations later
 			if (sig == 0)
 				return _ex0;
@@ -317,6 +319,20 @@ ex indexed::eval(int level) const
 	return ex_to<basic>(base).eval_indexed(*this);
 }
 
+ex indexed::real_part() const
+{
+	if(op(0).info(info_flags::real))
+		return *this;
+	return real_part_function(*this).hold();
+}
+
+ex indexed::imag_part() const
+{
+	if(op(0).info(info_flags::real))
+		return 0;
+	return imag_part_function(*this).hold();
+}
+
 ex indexed::thiscontainer(const exvector & v) const
 {
 	return indexed(ex_to<symmetry>(symtree), v);
@@ -327,6 +343,14 @@ ex indexed::thiscontainer(std::auto_ptr<exvector> vp) const
 	return indexed(ex_to<symmetry>(symtree), vp);
 }
 
+unsigned indexed::return_type() const
+{
+	if(is_a<matrix>(op(0)))
+		return return_types::commutative;
+	else
+		return op(0).return_type();
+}
+
 ex indexed::expand(unsigned options) const
 {
 	GINAC_ASSERT(seq.size() > 0);
@@ -512,22 +536,6 @@ struct is_summation_idx : public std::unary_function<ex, bool> {
 	}
 };
 
-exvector power::get_free_indices() const
-{
-	// Get free indices of basis
-	exvector basis_indices = basis.get_free_indices();
-
-	if (exponent.info(info_flags::even)) {
-		// If the exponent is an even number, then any "free" index that
-		// forms a dummy pair with itself is actually a summation index
-		exvector really_free;
-		std::remove_copy_if(basis_indices.begin(), basis_indices.end(),
-		                    std::back_inserter(really_free), is_summation_idx());
-		return really_free;
-	} else
-		return basis_indices;
-}
-
 exvector integral::get_free_indices() const
 {
 	if (a.get_free_indices().size() || b.get_free_indices().size())
@@ -774,9 +782,6 @@ static void product_to_exvector(const ex & e, exvector & v, bool & non_commutati
 	}
 }
 
-// Forward declaration needed in absence of friend injection, C.f. [namespace.memdef]:
-ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indices, const scalar_products & sp);
-
 template<class T> ex idx_symmetrization(const ex& r,const exvector& local_dummy_indices)
 {	exvector dummy_syms;
 	dummy_syms.reserve(r.nops());
@@ -789,6 +794,9 @@ template<class T> ex idx_symmetrization(const ex& r,const exvector& local_dummy_
 	return q;
 }
 
+// Forward declaration needed in absence of friend injection, C.f. [namespace.memdef]:
+ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indices, const scalar_products & sp);
+
 /** Simplify product of indexed expressions (commutative, noncommutative and
  *  simple squares), return list of free indices. */
 ex simplify_indexed_product(const ex & e, exvector & free_indices, exvector & dummy_indices, const scalar_products & sp)
@@ -1391,6 +1399,46 @@ void scalar_products::debugprint() const
 	}
 }
 
+exvector get_all_dummy_indices_safely(const ex & e)
+{
+	if (is_a<indexed>(e))
+		return ex_to<indexed>(e).get_dummy_indices();
+	else if (is_a<power>(e) && e.op(1)==2) {
+		return e.op(0).get_free_indices();
+	}	
+	else if (is_a<mul>(e) || is_a<ncmul>(e)) {
+		exvector dummies;
+		exvector free_indices;
+		for (int i=0; i<e.nops(); ++i) {
+			exvector dummies_of_factor = get_all_dummy_indices_safely(e.op(i));
+			dummies.insert(dummies.end(), dummies_of_factor.begin(),
+				dummies_of_factor.end());
+			exvector free_of_factor = e.op(i).get_free_indices();
+			free_indices.insert(free_indices.begin(), free_of_factor.begin(),
+				free_of_factor.end());
+		}
+		exvector free_out, dummy_out;
+		find_free_and_dummy(free_indices.begin(), free_indices.end(), free_out,
+			dummy_out);
+		dummies.insert(dummies.end(), dummy_out.begin(), dummy_out.end());
+		return dummies;
+	}
+	else if(is_a<add>(e)) {
+		exvector result;
+		for(int i=0; i<e.nops(); ++i) {
+			exvector dummies_of_term = get_all_dummy_indices_safely(e.op(i));
+			sort(dummies_of_term.begin(), dummies_of_term.end());
+			exvector new_vec;
+			set_union(result.begin(), result.end(), dummies_of_term.begin(),
+				dummies_of_term.end(), std::back_inserter<exvector>(new_vec),
+				ex_is_less());
+			result.swap(new_vec);
+		}
+		return result;
+	}
+	return exvector();
+}
+
 /** Returns all dummy indices from the exvector */
 exvector get_all_dummy_indices(const ex & e)
 {
@@ -1417,34 +1465,93 @@ exvector get_all_dummy_indices(const ex & e)
 	return v;
 }
 
-ex rename_dummy_indices_uniquely(const ex & a, const ex & b)
+lst rename_dummy_indices_uniquely(const exvector & va, const exvector & vb)
 {
-	exvector va = get_all_dummy_indices(a), vb = get_all_dummy_indices(b), common_indices;
+	exvector common_indices;
 	set_intersection(va.begin(), va.end(), vb.begin(), vb.end(), std::back_insert_iterator<exvector>(common_indices), ex_is_less());
 	if (common_indices.empty()) {
-		return b;
+		return lst(lst(), lst());
 	} else {
 		exvector new_indices, old_indices;
 		old_indices.reserve(2*common_indices.size());
 		new_indices.reserve(2*common_indices.size());
 		exvector::const_iterator ip = common_indices.begin(), ipend = common_indices.end();
 		while (ip != ipend) {
-			if (is_a<varidx>(*ip)) {
-				varidx mu((new symbol)->setflag(status_flags::dynallocated), ex_to<varidx>(*ip).get_dim(), ex_to<varidx>(*ip).is_covariant());
-				old_indices.push_back(*ip);
-				new_indices.push_back(mu);
+			ex newsym=(new symbol)->setflag(status_flags::dynallocated);
+			ex newidx;
+			if(is_exactly_a<spinidx>(*ip))
+				newidx = (new spinidx(newsym, ex_to<spinidx>(*ip).get_dim(),
+						ex_to<spinidx>(*ip).is_covariant(),
+						ex_to<spinidx>(*ip).is_dotted()))
+					-> setflag(status_flags::dynallocated);
+			else if (is_exactly_a<varidx>(*ip))
+				newidx = (new varidx(newsym, ex_to<varidx>(*ip).get_dim(),
+						ex_to<varidx>(*ip).is_covariant()))
+					-> setflag(status_flags::dynallocated);
+			else
+				newidx = (new idx(newsym, ex_to<idx>(*ip).get_dim()))
+					-> setflag(status_flags::dynallocated);
+			old_indices.push_back(*ip);
+			new_indices.push_back(newidx);
+			if(is_a<varidx>(*ip)) {
 				old_indices.push_back(ex_to<varidx>(*ip).toggle_variance());
-				new_indices.push_back(mu.toggle_variance());
-			} else {
-				old_indices.push_back(*ip);
-				new_indices.push_back(idx((new symbol)->setflag(status_flags::dynallocated), ex_to<varidx>(*ip).get_dim()));
+				new_indices.push_back(ex_to<varidx>(newidx).toggle_variance());
 			}
 			++ip;
 		}
-		return b.subs(lst(old_indices.begin(), old_indices.end()), lst(new_indices.begin(), new_indices.end()), subs_options::no_pattern);
+		return lst(lst(old_indices.begin(), old_indices.end()), lst(new_indices.begin(), new_indices.end()));
 	}
 }
 
+ex rename_dummy_indices_uniquely(const exvector & va, const exvector & vb, const ex & b)
+{
+	lst indices_subs = rename_dummy_indices_uniquely(va, vb);
+	return (indices_subs.op(0).nops()>0 ? b.subs(ex_to<lst>(indices_subs.op(0)), ex_to<lst>(indices_subs.op(1)), subs_options::no_pattern|subs_options::no_index_renaming) : b);
+}
+
+ex rename_dummy_indices_uniquely(const ex & a, const ex & b)
+{
+	exvector va = get_all_dummy_indices_safely(a);
+	if (va.size() > 0) {
+		exvector vb = get_all_dummy_indices_safely(b);
+		if (vb.size() > 0) {
+			sort(va.begin(), va.end(), ex_is_less());
+			sort(vb.begin(), vb.end(), ex_is_less());
+			lst indices_subs = rename_dummy_indices_uniquely(va, vb);
+			if (indices_subs.op(0).nops() > 0)
+				return b.subs(ex_to<lst>(indices_subs.op(0)), ex_to<lst>(indices_subs.op(1)), subs_options::no_pattern|subs_options::no_index_renaming);
+		}
+	}
+	return b;
+}
+
+ex rename_dummy_indices_uniquely(exvector & va, const ex & b, bool modify_va)
+{
+	if (va.size() > 0) {
+		exvector vb = get_all_dummy_indices_safely(b);
+		if (vb.size() > 0) {
+			sort(vb.begin(), vb.end(), ex_is_less());
+			lst indices_subs = rename_dummy_indices_uniquely(va, vb);
+			if (indices_subs.op(0).nops() > 0) {
+				if (modify_va) {
+					for (lst::const_iterator i = ex_to<lst>(indices_subs.op(1)).begin(); i != ex_to<lst>(indices_subs.op(1)).end(); ++i)
+						va.push_back(*i);
+					exvector uncommon_indices;
+					set_difference(vb.begin(), vb.end(), indices_subs.op(0).begin(), indices_subs.op(0).end(), std::back_insert_iterator<exvector>(uncommon_indices), ex_is_less());
+					exvector::const_iterator ip = uncommon_indices.begin(), ipend = uncommon_indices.end();
+					while (ip != ipend) {
+						va.push_back(*ip);
+						++ip;
+					}
+					sort(va.begin(), va.end(), ex_is_less());
+				}
+				return b.subs(ex_to<lst>(indices_subs.op(0)), ex_to<lst>(indices_subs.op(1)), subs_options::no_pattern|subs_options::no_index_renaming);
+			}
+		}
+	}
+	return b;
+}
+
 ex expand_dummy_sum(const ex & e, bool subs_idx)
 {
 	ex e_expanded = e.expand();
