@@ -3,7 +3,7 @@
  *  Interface to GiNaC's products of expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,8 +41,8 @@ public:
 	mul(const ex & lh, const ex & rh);
 	mul(const exvector & v);
 	mul(const epvector & v);
-	mul(const epvector & v, const ex & oc);
-	mul(std::auto_ptr<epvector> vp, const ex & oc);
+	mul(const epvector & v, const ex & oc, bool do_index_renaming = false);
+	mul(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming = false);
 	mul(const ex & lh, const ex & mh, const ex & rh);
 	
 	// functions overriding virtual functions from base classes
@@ -52,8 +52,11 @@ public:
 	int degree(const ex & s) const;
 	int ldegree(const ex & s) const;
 	ex coeff(const ex & s, int n = 1) const;
+	bool has(const ex & other, unsigned options = 0) const;
 	ex eval(int level=0) const;
 	ex evalf(int level=0) const;
+	ex real_part() const;
+	ex imag_part() const;
 	ex evalm() const;
 	ex series(const relational & s, int order, unsigned options = 0) const;
 	ex normal(exmap & repl, exmap & rev_lookup, int level = 0) const;
@@ -65,9 +68,9 @@ protected:
 	ex derivative(const symbol & s) const;
 	ex eval_ncmul(const exvector & v) const;
 	unsigned return_type() const;
-	unsigned return_type_tinfo() const;
-	ex thisexpairseq(const epvector & v, const ex & oc) const;
-	ex thisexpairseq(std::auto_ptr<epvector> vp, const ex & oc) const;
+	tinfo_t return_type_tinfo() const;
+	ex thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming = false) const;
+	ex thisexpairseq(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming = false) const;
 	expair split_ex_to_pair(const ex & e) const;
 	expair combine_ex_with_coeff_to_pair(const ex & e, const ex & c) const;
 	expair combine_pair_with_coeff_to_pair(const expair & p, const ex & c) const;
@@ -78,6 +81,7 @@ protected:
 	void combine_overall_coeff(const ex & c1, const ex & c2);
 	bool can_make_flat(const expair & p) const;
 	ex expand(unsigned options=0) const;
+	void find_real_imag(ex&, ex&) const;
 	
 	// new virtual functions which can be overridden by derived classes
 	// none
@@ -95,14 +99,6 @@ protected:
 	std::auto_ptr<epvector> expandchildren(unsigned options) const;
 };
 
-// utility functions
-
-/** Specialization of is_exactly_a<mul>(obj) for mul objects. */
-template<> inline bool is_exactly_a<mul>(const basic & obj)
-{
-	return obj.tinfo()==TINFO_mul;
-}
-
 } // namespace GiNaC
 
 #endif // ndef __GINAC_MUL_H__
