@@ -27,6 +27,7 @@
 #include "ex.h"
 
 #include <stdexcept>
+#include <vector>
 
 #include <cln/complex.h>
 
@@ -39,6 +40,12 @@
 
 namespace GiNaC {
 
+/** Function pointer to implement callbacks in the case 'Digits' gets changed.
+ *  Main purpose of such callbacks is to adjust look-up tables of certain
+ *  functions to the new precision. Parameter contains the signed difference
+ *  between new Digits and old Digits. */
+typedef void (* digits_changed_callback)(long);
+
 /** This class is used to instantiate a global singleton object Digits
  *  which behaves just like Maple's Digits.  We need an object rather 
  *  than a dumber basic type since as a side-effect we let it change
@@ -55,11 +62,14 @@ public:
 	_numeric_digits();
 	_numeric_digits& operator=(long prec);
 	operator long();
-	void print(std::ostream &os) const;
+	void print(std::ostream& os) const;
+	void add_callback(digits_changed_callback callback);
 // member variables
 private:
 	long digits;                        ///< Number of decimal digits
 	static bool too_late;               ///< Already one object present
+	// Holds a list of functions that get called when digits is changed.
+	std::vector<digits_changed_callback> callbacklist;
 };
 
 
@@ -95,10 +105,11 @@ public:
 public:
 	unsigned precedence() const {return 30;}
 	bool info(unsigned inf) const;
+	bool is_polynomial(const ex & var) const;
 	int degree(const ex & s) const;
 	int ldegree(const ex & s) const;
 	ex coeff(const ex & s, int n = 1) const;
-	bool has(const ex &other) const;
+	bool has(const ex &other, unsigned options = 0) const;
 	ex eval(int level = 0) const;
 	ex evalf(int level = 0) const;
 	ex subs(const exmap & m, unsigned options = 0) const { return subs_one_level(m, options); } // overwrites basic::subs() for performance reasons
@@ -109,6 +120,8 @@ public:
 	ex smod(const numeric &xi) const;
 	numeric max_coefficient() const;
 	ex conjugate() const;
+	ex real_part() const;
+	ex imag_part() const;
 protected:
 	/** Implementation of ex::diff for a numeric always returns 0.
 	 *  @see ex::diff */
@@ -138,6 +151,7 @@ public:
 	const numeric & operator=(double d);
 	const numeric & operator=(const char *s);
 	const numeric inverse() const;
+	numeric step() const;
 	int csgn() const;
 	int compare(const numeric &other) const;
 	bool is_equal(const numeric &other) const;
@@ -240,6 +254,9 @@ inline const numeric pow(const numeric &x, const numeric &y)
 inline const numeric inverse(const numeric &x)
 { return x.inverse(); }
 
+inline numeric step(const numeric &x)
+{ return x.step(); }
+
 inline int csgn(const numeric &x)
 { return x.csgn(); }
 
@@ -249,6 +266,9 @@ inline bool is_zero(const numeric &x)
 inline bool is_positive(const numeric &x)
 { return x.is_positive(); }
 
+inline bool is_negative(const numeric &x)
+{ return x.is_negative(); }
+
 inline bool is_integer(const numeric &x)
 { return x.is_integer(); }
 
@@ -307,14 +327,6 @@ ex EulerEvalf();
 ex CatalanEvalf();
 
 
-// utility functions
-
-/** Specialization of is_exactly_a<numeric>(obj) for numeric objects. */
-template<> inline bool is_exactly_a<numeric>(const basic & obj)
-{
-	return obj.tinfo()==TINFO_numeric;
-}
-
 } // namespace GiNaC
 
 #ifdef __MAKECINT__
