@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's overloaded operators. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,10 +48,11 @@ static inline const ex exmul(const ex & lh, const ex & rh)
 	// ncmul::eval()'s rule to pull out commutative elements we need to check
 	// only one of the elements.
 	if (rh.return_type()==return_types::commutative ||
-	    lh.return_type()==return_types::commutative)
+	    lh.return_type()==return_types::commutative) {
 		return (new mul(lh,rh))->setflag(status_flags::dynallocated);
-	else
+	} else {
 		return (new ncmul(lh,rh))->setflag(status_flags::dynallocated);
+	}
 }
 
 /** Used internally by operator-() and friends to change the sign of an argument. */
@@ -349,6 +350,92 @@ std::ostream & operator<<(std::ostream & os, const ex & e)
 	return os;
 }
 
+std::ostream & operator<<(std::ostream & os, const exvector & e)
+{
+	print_context *p = get_print_context(os);
+	exvector::const_iterator i = e.begin();
+	exvector::const_iterator vend = e.end();
+
+	if (i==vend) {
+		os << "[]";
+		return os;
+	}
+
+	os << "[";
+	while (true) {
+		if (p == 0)
+			i -> print(print_dflt(os));
+		else
+			i -> print(*p);
+		++i;
+		if (i==vend)
+			break;
+		os << ",";
+	}
+	os << "]";
+
+	return os;
+}
+
+std::ostream & operator<<(std::ostream & os, const exset & e)
+{
+	print_context *p = get_print_context(os);
+	exset::const_iterator i = e.begin();
+	exset::const_iterator send = e.end();
+
+	if (i==send) {
+		os << "<>";
+		return os;
+	}
+
+	os << "<";
+	while (true) {
+		if (p == 0)
+			i->print(print_dflt(os));
+		else
+			i->print(*p);
+		++i;
+		if (i == send)
+			break;
+		os << ",";
+	}
+	os << ">";
+
+	return os;
+}
+
+std::ostream & operator<<(std::ostream & os, const exmap & e)
+{
+	print_context *p = get_print_context(os);
+	exmap::const_iterator i = e.begin();
+	exmap::const_iterator mend = e.end();
+
+	if (i==mend) {
+		os << "{}";
+		return os;
+	}
+
+	os << "{";
+	while (true) {
+		if (p == 0)
+			i->first.print(print_dflt(os));
+		else
+			i->first.print(*p);
+		os << "==";
+		if (p == 0)
+			i->second.print(print_dflt(os));
+		else
+			i->second.print(*p);
+		++i;
+		if( i==mend )
+			break;
+		os << ",";
+	}
+	os << "}";
+
+	return os;
+}
+
 std::istream & operator>>(std::istream & is, ex & e)
 {
 	throw (std::logic_error("expression input from streams not implemented"));
