@@ -3,7 +3,7 @@
  *  GiNaC's class registrar (for class basic and all classes derived from it). */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 
 namespace GiNaC {
 
-unsigned find_tinfo_key(const std::string &class_name)
+tinfo_t find_tinfo_key(const std::string &class_name)
 {
 	return registered_class_info::find(class_name)->options.get_id();
 }
