@@ -33,8 +33,6 @@
 
 namespace GiNaC {
 
-extern unsigned next_structure_tinfo_key;
-
 
 /** Comparison policy: all structures of one type are equal */
 template <class T>
@@ -119,8 +117,9 @@ class structure : public basic, public ComparisonPolicy<T> {
 	GINAC_DECLARE_REGISTERED_CLASS(structure, basic)
 
 	// helpers
-	static unsigned get_tinfo() { return reg_info.options.get_id(); }
+	static tinfo_t get_tinfo() { return reg_info.options.get_id(); }
 	static const char *get_class_name() { return "structure"; }
+	static tinfo_t next_structure_tinfo_key() { return new tinfo_static_t; }
 
 	// constructors
 public:
@@ -156,7 +155,7 @@ public:
 	ex & operator[](size_t i) { return inherited::operator[](i); }
 
 	// pattern matching
-	bool has(const ex & other) const { return inherited::has(other); }
+	bool has(const ex & other, unsigned options = 0) const { return inherited::has(other, options); }
 	bool match(const ex & pattern, lst & repl_lst) const { return inherited::match(pattern, repl_lst); }
 protected:
 	bool match_same_type(const basic & other) const { return true; }
@@ -201,7 +200,7 @@ public:
 
 	// noncommutativity
 	unsigned return_type() const { return return_types::commutative; }
-	unsigned return_type_tinfo() const { return tinfo(); }
+	tinfo_t return_type_tinfo() const { return this; }
 
 protected:
 	bool is_equal_same_type(const basic & other) const
@@ -220,7 +219,6 @@ public:
 	const T *operator->() const { return &obj; }
 	T &get_struct() { return obj; }
 	const T &get_struct() const { return obj; }
-
 private:
 	T obj;
 };
@@ -259,7 +257,7 @@ int structure<T, CP>::compare_same_type(const basic & other) const
 }
 
 template <class T, template <class> class CP>
-registered_class_info structure<T, CP>::reg_info = registered_class_info(registered_class_options(structure::get_class_name(), "basic", next_structure_tinfo_key++, &structure::unarchive));
+registered_class_info structure<T, CP>::reg_info = registered_class_info(registered_class_options(structure::get_class_name(), "basic", structure::next_structure_tinfo_key(), &structure::unarchive));
 
 
 } // namespace GiNaC
