@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's symmetry definitions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <iostream>
 #include <stdexcept>
 #include <functional>
+#include <limits>
 
 #include "symmetry.h"
 #include "lst.h"
@@ -55,7 +56,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(symmetry, basic,
 // default constructor
 //////////
 
-symmetry::symmetry() : inherited(TINFO_symmetry), type(none)
+symmetry::symmetry() : inherited(&symmetry::tinfo_static), type(none)
 {
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
@@ -64,13 +65,13 @@ symmetry::symmetry() : inherited(TINFO_symmetry), type(none)
 // other constructors
 //////////
 
-symmetry::symmetry(unsigned i) : inherited(TINFO_symmetry), type(none)
+symmetry::symmetry(unsigned i) : inherited(&symmetry::tinfo_static), type(none)
 {
 	indices.insert(i);
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
 
-symmetry::symmetry(symmetry_type t, const symmetry &c1, const symmetry &c2) : inherited(TINFO_symmetry), type(t)
+symmetry::symmetry(symmetry_type t, const symmetry &c1, const symmetry &c2) : inherited(&symmetry::tinfo_static), type(t)
 {
 	add(c1); add(c2);
 	setflag(status_flags::evaluated | status_flags::expanded);
@@ -142,11 +143,69 @@ int symmetry::compare_same_type(const basic & other) const
 {
 	GINAC_ASSERT(is_a<symmetry>(other));
 
-	// All symmetry trees are equal. They are not supposed to appear in
-	// ordinary expressions anyway...
+	// For archiving purposes we need to have an ordering of symmetries.
+	const symmetry &othersymm = ex_to<symmetry>(other);
+
+	// Compare type.
+	if (type > othersymm.type)
+		return 1;
+	if (type < othersymm.type)
+		return -1;
+
+	// Compare the index set.
+	size_t this_size = indices.size();
+	size_t that_size = othersymm.indices.size();
+	if (this_size > that_size)
+		return 1;
+	if (this_size < that_size)
+		return -1;
+	typedef std::set<unsigned>::iterator set_it;
+	set_it end = indices.end();
+	for (set_it i=indices.begin(),j=othersymm.indices.begin(); i!=end; ++i,++j) {
+		if(*i < *j)
+			return 1;
+		if(*i > *j)
+			return -1;
+	}
+
+	// Compare the children.
+	if (children.size() > othersymm.children.size())
+		return 1;
+	if (children.size() < othersymm.children.size())
+		return -1;
+	for (size_t i=0; i<children.size(); ++i) {
+		int cmpval = ex_to<symmetry>(children[i])
+			.compare_same_type(ex_to<symmetry>(othersymm.children[i]));
+		if (cmpval)
+			return cmpval;
+	}
+
 	return 0;
 }
 
+unsigned symmetry::calchash() const
+{
+	unsigned v = golden_ratio_hash((p_int)tinfo());
+
+	if (type == none) {
+		v = rotate_left(v);
+		v ^= *(indices.begin());
+	} else {
+		for (exvector::const_iterator i=children.begin(); i!=children.end(); ++i)
+		{
+			v = rotate_left(v);
+			v ^= i->gethash();
+		}
+	}
+
+	if (flags & status_flags::evaluated) {
+		setflag(status_flags::hash_calculated);
+		hashvalue = v;
+	}
+
+	return v;
+}
+
 void symmetry::do_print(const print_context & c, unsigned level) const
 {
 	if (children.empty()) {
@@ -208,6 +267,18 @@ void symmetry::do_print_tree(const print_tree & c, unsigned level) const
 // non-virtual functions in this class
 //////////
 
+bool symmetry::has_cyclic() const
+{
+	if (type == cyclic)
+		return true;
+
+	for (exvector::const_iterator i=children.begin(); i!=children.end(); ++i)
+		if (ex_to<symmetry>(*i).has_cyclic())
+			return true;
+
+	return false;
+}
+
 symmetry &symmetry::add(const symmetry &c)
 {
 	// All children must have the same number of indices
@@ -361,7 +432,7 @@ int canonicalize(exvector::iterator v, const symmetry &symm)
 {
 	// Less than two elements? Then do nothing
 	if (symm.indices.size() < 2)
-		return INT_MAX;
+		return std::numeric_limits<int>::max();
 
 	// Canonicalize children first
 	bool something_changed = false;
@@ -372,7 +443,7 @@ int canonicalize(exvector::iterator v, const symmetry &symm)
 		int child_sign = canonicalize(v, ex_to<symmetry>(*first));
 		if (child_sign == 0)
 			return 0;
-		if (child_sign != INT_MAX) {
+		if (child_sign != std::numeric_limits<int>::max()) {
 			something_changed = true;
 			sign *= child_sign;
 		}
@@ -399,7 +470,7 @@ int canonicalize(exvector::iterator v, const symmetry &symm)
 		default:
 			break;
 	}
-	return something_changed ? sign : INT_MAX;
+	return something_changed ? sign : std::numeric_limits<int>::max();
 }
 
 
@@ -427,7 +498,7 @@ static ex symm(const ex & e, exvector::const_iterator first, exvector::const_ite
 		lst new_lst;
 		for (unsigned i=0; i<num; i++)
 			new_lst.append(orig_lst.op(iv[i]));
-		ex term = e.subs(orig_lst, new_lst, subs_options::no_pattern);
+		ex term = e.subs(orig_lst, new_lst, subs_options::no_pattern|subs_options::no_index_renaming);
 		if (asymmetric) {
 			memcpy(iv2, iv, num * sizeof(unsigned));
 			term *= permutation_sign(iv2, iv2 + num);
@@ -468,7 +539,7 @@ ex symmetrize_cyclic(const ex & e, exvector::const_iterator first, exvector::con
 	for (unsigned i=0; i<num-1; i++) {
 		ex perm = new_lst.op(0);
 		new_lst.remove_first().append(perm);
-		sum += e.subs(orig_lst, new_lst, subs_options::no_pattern);
+		sum += e.subs(orig_lst, new_lst, subs_options::no_pattern|subs_options::no_index_renaming);
 	}
 	return sum / num;
 }
