@@ -64,7 +64,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(tensepsilon, tensor,
 // constructors
 //////////
 
-tensor::tensor() : inherited(TINFO_tensor)
+tensor::tensor() : inherited(&tensor::tinfo_static)
 {
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
@@ -74,27 +74,27 @@ DEFAULT_CTOR(tensmetric)
 
 minkmetric::minkmetric() : pos_sig(false)
 {
-	tinfo_key = TINFO_minkmetric;
+	tinfo_key = &minkmetric::tinfo_static;
 }
 
 spinmetric::spinmetric()
 {
-	tinfo_key = TINFO_spinmetric;
+	tinfo_key = &spinmetric::tinfo_static;
 }
 
 minkmetric::minkmetric(bool ps) : pos_sig(ps)
 {
-	tinfo_key = TINFO_minkmetric;
+	tinfo_key = &minkmetric::tinfo_static;
 }
 
 tensepsilon::tensepsilon() : minkowski(false), pos_sig(false)
 {
-	tinfo_key = TINFO_tensepsilon;
+	tinfo_key = &tensepsilon::tinfo_static;
 }
 
 tensepsilon::tensepsilon(bool mink, bool ps) : minkowski(mink), pos_sig(ps)
 {
-	tinfo_key = TINFO_tensepsilon;
+	tinfo_key = &tensepsilon::tinfo_static;
 }
 
 //////////
@@ -141,6 +141,22 @@ DEFAULT_COMPARE(tensdelta)
 DEFAULT_COMPARE(tensmetric)
 DEFAULT_COMPARE(spinmetric)
 
+bool tensdelta::info(unsigned inf) const
+{
+	if(inf == info_flags::real)
+		return true;
+
+	return false;
+}
+
+bool tensmetric::info(unsigned inf) const
+{
+	if(inf == info_flags::real)
+		return true;
+
+	return false;
+}
+
 int minkmetric::compare_same_type(const basic & other) const
 {
 	GINAC_ASSERT(is_a<minkmetric>(other));
@@ -152,6 +168,14 @@ int minkmetric::compare_same_type(const basic & other) const
 		return inherited::compare_same_type(other);
 }
 
+bool minkmetric::info(unsigned inf) const
+{
+	if(inf == info_flags::real)
+		return true;
+
+	return false;
+}
+
 int tensepsilon::compare_same_type(const basic & other) const
 {
 	GINAC_ASSERT(is_a<tensepsilon>(other));
@@ -165,6 +189,22 @@ int tensepsilon::compare_same_type(const basic & other) const
 		return inherited::compare_same_type(other);
 }
 
+bool tensepsilon::info(unsigned inf) const
+{
+	if(inf == info_flags::real)
+		return true;
+
+	return false;
+}
+
+bool spinmetric::info(unsigned inf) const
+{
+	if(inf == info_flags::real)
+		return true;
+
+	return false;
+}
+
 DEFAULT_PRINT_LATEX(tensdelta, "delta", "\\delta")
 DEFAULT_PRINT(tensmetric, "g")
 DEFAULT_PRINT_LATEX(minkmetric, "eta", "\\eta")
@@ -590,6 +630,9 @@ ex epsilon_tensor(const ex & i1, const ex & i2)
 	if (!ex_to<idx>(i1).get_dim().is_equal(_ex2))
 		throw(std::runtime_error("index dimension of epsilon tensor must match number of indices"));
 
+	if(is_a<wildcard>(i1.op(0))||is_a<wildcard>(i2.op(0)))
+		return indexed(epsilon, antisymmetric2(), i1, i2).hold();
+
 	return indexed(epsilon, antisymmetric2(), i1, i2);
 }
 
@@ -606,6 +649,9 @@ ex epsilon_tensor(const ex & i1, const ex & i2, const ex & i3)
 	if (!ex_to<idx>(i1).get_dim().is_equal(_ex3))
 		throw(std::runtime_error("index dimension of epsilon tensor must match number of indices"));
 
+	if(is_a<wildcard>(i1.op(0))||is_a<wildcard>(i2.op(0))||is_a<wildcard>(i3.op(0)))
+		return indexed(epsilon, antisymmetric3(), i1, i2, i3).hold();
+
 	return indexed(epsilon, antisymmetric3(), i1, i2, i3);
 }
 
@@ -623,6 +669,9 @@ ex lorentz_eps(const ex & i1, const ex & i2, const ex & i3, const ex & i4, bool
 	if (!ex_to<idx>(i1).get_dim().is_equal(_ex4))
 		throw(std::runtime_error("index dimension of epsilon tensor must match number of indices"));
 
+	if(is_a<wildcard>(i1.op(0))||is_a<wildcard>(i2.op(0))||is_a<wildcard>(i3.op(0))||is_a<wildcard>(i4.op(0)))
+		return indexed(pos_sig ? epsilon_pos : epsilon_neg, antisymmetric4(), i1, i2, i3, i4).hold();
+
 	return indexed(pos_sig ? epsilon_pos : epsilon_neg, antisymmetric4(), i1, i2, i3, i4);
 }
 
